@@ -8,6 +8,9 @@
 /*
  * Initialize the exceptionFrame object "f" with the stack frame captured in
  * the given sigcontext structure.
+ *
+ * from ppc/signal.h sigcontext definition:
+ void    *sc_regs;                (kernel private) saved state 
  */
 #define EXCEPTIONFRAME(f, c) \
 do { \
