@@ -13,12 +13,14 @@
  * of this file.
  */
 
+#include <stdlib.h>
+
 #include "config.h"
 #include "config-std.h"
 #include "config-mem.h"
 #include "debug.h"
 #include "classMethod.h"
-#include "kaffe/jni_md.h"
+#include "jni_md.h"
 #include "native.h"
 #include "constants.h"
 #include "support.h"
@@ -34,7 +36,6 @@
 #if defined(KAFFE_FEEDBACK)
 #include "feedback.h"
 #endif
-#include "methodCache.h"
 #include "external.h"
 #include "kaffe_jni.h"
 
@@ -49,7 +50,7 @@
 #endif
 
 #ifdef ENABLE_BINRELOC
-#include "prefix.h"
+#include "binreloc.h"
 #endif
 
 #if defined(KAFFE_PROFILER)
@@ -59,15 +60,13 @@ extern int profFlag;
 #include "jni.h"
 
 KaffeVM_Arguments vmargs;
-JNIEnv* global_env;
 JavaVM* global_vm;
 static int isJar = 0;
-static char *jvm_onload;
 
 static int options(char**, int);
 static void usage(void);
 static size_t parseSize(char*);
-static int checkException(void);
+static int checkException(JNIEnv* env);
 static int main2(JNIEnv* env, char *argv[], int farg, int argc);
 
 #define	KAFFEHOME	"KAFFEHOME"
@@ -87,11 +86,21 @@ main(int argc, char* argv[])
 {
 	int farg;
 	const char* cp;
+	void* env;
 
 #if defined(MAIN_MD)
 	MAIN_MD;
 #endif
 
+#if defined(ENABLE_BINRELOC)
+        BrInitError error;
+
+        if( br_init(&error) == 0 && error != BR_INIT_ERROR_DISABLED) {
+                printf ("Warning: BinReloc failed to initialize (error code %d)\n", error);
+                printf ("Will fallback to hardcoded default path.\n");
+        };
+#endif
+
 #if defined(HAVE_LC_MESSAGES)
 	setlocale(LC_MESSAGES, "");
 	setlocale(LC_CTYPE, "");
@@ -130,7 +139,7 @@ main(int argc, char* argv[])
 		}
 #endif
 	}
-	vmargs.classpath = cp;
+	vmargs.classpath = (cp == NULL? NULL :strdup(cp));
 
         cp = getenv(LIBRARYPATH1);
 	if (cp == 0) {
@@ -166,60 +175,27 @@ main(int argc, char* argv[])
 	/* Get the class name to start with */
 	if (argv[farg] == 0) {
 		usage();
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	if (strcmp(argv[farg] + strlen(argv[farg]) - strlen(".class"),
 		   ".class") == 0) {
 		fprintf(stderr,
 			"Please do not specify the .class extension\n");
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	/* Initialise */
-	JNI_CreateJavaVM(&global_vm, &global_env, &vmargs);
+	if (JNI_CreateJavaVM(&global_vm, 
+			     &env, 
+			     &vmargs) 
+	    < 0)
+	  {
+	    fprintf(stderr, "Cannot create the Java VM\n");
+	    exit(EXIT_FAILURE);
+	  }
 
-	/* Handle the '-Xrun' argument. */
-	if( jvm_onload != NULL )
-	{
-		char *libpath, *libargs;
-		char errbuf[512];
-		int i;
-
-		/* XXX Pull findLibrary() from the JanosVM. */
-		libpath = &jvm_onload[2];
-		libpath[0] = 'l';
-		libpath[1] = 'i';
-		libpath[2] = 'b';
-
-		if( (libargs = strchr(jvm_onload, ':')) != NULL )
-		{
-			*libargs = '\0';
-			libargs += 1;
-		}
-		
-		i = loadNativeLibrary(libpath, NULL, errbuf, sizeof(errbuf));
-		if( i > 0 )
-		{
-			jint (*onload_func)(JavaVM *jvm, char *, void *);
-
-			if( (onload_func =
-			     loadNativeLibrarySym("JVM_OnLoad")) != NULL )
-			{
-				(void)onload_func(global_vm, libargs, NULL);
-			}
-		}
-		else
-		{
-			fprintf(stderr,
-				"Unable to load %s: %s\n",
-				libpath,
-				errbuf);
-			exit(1);
-		}
-	}
-
-	return main2(global_env, argv, farg, argc);
+	return main2(env, argv, farg, argc);
 }
 
 /*
@@ -273,7 +249,7 @@ main2(JNIEnv* env, char *argv[], int farg, int argc)
 
 	/* Executable is a JAR?  Use the JAR launcher */
 	if (isJar != 0) {
-		exec = "kaffe.jar.ExecJar";
+		exec = "org.kaffe.jar.ExecJar";
 	}
 	else {
 		exec = argv[farg];
@@ -282,34 +258,34 @@ main2(JNIEnv* env, char *argv[], int farg, int argc)
 	}
 	
 	mcls = (*env)->FindClass(env, exec);
-	if (checkException())
+	if (checkException(env))
 		goto exception_happened;
 	
 	/* ... and run main. */
 	mmth = (*env)->GetStaticMethodID(env,
 	    mcls, "main", "([Ljava/lang/String;)V");
-	if (checkException())
+	if (checkException(env))
 		goto exception_happened;
 
 	/* Build an array of strings as the arguments */
 	cls = (*env)->FindClass(env, "java/lang/String");
-	if (checkException())
+	if (checkException(env))
 		goto exception_happened;
 	args = (*env)->NewObjectArray(env, argc, cls, NULL);
-	if (checkException())
+	if (checkException(env))
 		goto exception_happened;
 	for (i = 0; i < argc; i++) {
 		str = (*env)->NewStringUTF(env, argv[farg+i]);
-		if (checkException())
+		if (checkException(env))
 			goto exception_happened;
 		(*env)->SetObjectArrayElement(env, args, i, str);
-		if (checkException())
+		if (checkException(env))
 			goto exception_happened;
 	}
 
 	/* Call method, check for errors and then exit */
 	(*env)->CallStaticVoidMethod(env, mcls, mmth, args);
-	if (checkException())
+	if (checkException(env))
 	  goto exception_happened;
 
 	ret_code = 0;
@@ -325,35 +301,35 @@ done:
 }
 
 static int
-checkException(void)
+checkException(JNIEnv* env)
 {
 	jobject e;
 	jclass eiic;
 
 	/* Display exception stack trace */
-	if ((e = (*global_env)->ExceptionOccurred(global_env)) == NULL)
+	if ((e = (*env)->ExceptionOccurred(env)) == NULL)
 		return (0);
-	(*global_env)->DeleteLocalRef(global_env, e);
-	(*global_env)->ExceptionDescribe(global_env);
-	(*global_env)->ExceptionClear(global_env);
+	(*env)->DeleteLocalRef(env, e);
+	(*env)->ExceptionDescribe(env);
+	(*env)->ExceptionClear(env);
 
 	/* Display inner exception in ExceptionInInitializerError case */
-	eiic = (*global_env)->FindClass(global_env, "java/lang/ExceptionInInitializerError");
-	if ((*global_env)->ExceptionOccurred(global_env) != NULL) {
-		(*global_env)->ExceptionClear(global_env);
+	eiic = (*env)->FindClass(env, "java/lang/ExceptionInInitializerError");
+	if ((*env)->ExceptionOccurred(env) != NULL) {
+		(*env)->ExceptionClear(env);
 		return (1);
 	}
-	if ((*global_env)->IsInstanceOf(global_env, e, eiic)) {
-		e = (*global_env)->CallObjectMethod(global_env, e,
-		    (*global_env)->GetMethodID(global_env, (*global_env)->GetObjectClass(global_env, e),
+	if ((*env)->IsInstanceOf(env, e, eiic)) {
+		e = (*env)->CallObjectMethod(env, e,
+		    (*env)->GetMethodID(env, (*env)->GetObjectClass(env, e),
 			"getException", "()Ljava/lang/Throwable;"));
-		if ((*global_env)->ExceptionOccurred(global_env) != NULL) {
-			(*global_env)->ExceptionClear(global_env);
+		if ((*env)->ExceptionOccurred(env) != NULL) {
+			(*env)->ExceptionClear(env);
 			return (1);
 		}
 		if (e != NULL) {
-			(*global_env)->Throw(global_env, e);
-			return (checkException());
+			(*env)->Throw(env, e);
+			return (checkException(env));
 		}
 	}
 	return (1);
@@ -395,6 +371,9 @@ setUserProperty(char * propStr)
  * @param propStr AWT backend
  * @return updated user properties
  */
+
+#if defined (KAFFE_X_AWT_INCLUDED) || defined (KAFFE_QT_AWT_INCLUDED)	\
+  || defined (KAFFE_NANOX_AWT_INCLUDED)
 static 
 userProperty*
 setKaffeAWT(const char * propStr)
@@ -403,7 +382,7 @@ setKaffeAWT(const char * propStr)
 	unsigned int      bootcpathlength;
 	const char *prefix = 
 #if defined(ENABLE_BINRELOC)
-		LIBDIR
+		br_find_lib_dir(NULL)
 #else /* !defined(ENABLE_BINRELOC) */
 	        DEFAULT_KAFFEHOME
 #endif /* defined(ENABLE_BINRELOC) */
@@ -424,7 +403,7 @@ setKaffeAWT(const char * propStr)
        	if ((newbootcpath = malloc(bootcpathlength)) == NULL) 
 	{
            	fprintf(stderr,  _("Error: out of memory.\n"));
-                exit(1);
+                exit(EXIT_FAILURE);
       	}
 
         /* Construct new boot classpath */
@@ -442,9 +421,13 @@ setKaffeAWT(const char * propStr)
         /* select Xlib backend */
         prop = setUserProperty(backend_property);
 
+#if defined(ENABLE_BINRELOC)
+	free(prefix);
+#endif
+
 	return prop;
 }
-
+#endif
 /*
  * Process program's flags.
  */
@@ -464,15 +447,15 @@ options(char** argv, int argc)
 
 		if (strcmp(argv[i], "-help") == 0) {
 			usage();
-			exit(0);
+			exit(EXIT_SUCCESS);
 		}
 		else if (strcmp(argv[i], "-version") == 0) {
 			printShortVersion();
-			exit(0);
+			exit(EXIT_SUCCESS);
 		}
 		else if (strcmp(argv[i], "-fullversion") == 0) {
 			printFullVersion();
-			exit(0);
+			exit(EXIT_SUCCESS);
 		}
 #if defined(__ia64__)
 		else if (strcmp(argv[i], "-ia32") == 0) {
@@ -480,9 +463,21 @@ options(char** argv, int argc)
 			/* FIXME: skip, case handled by the calle script */
 		}
 #endif
-		else if ((strcmp(argv[i], "-addclasspath") == 0)
-			 || (strcmp(argv[i], "-classpath") == 0)
-			 || (strcmp(argv[i], "-cp") == 0)) {
+		else if (strcmp(argv[i], "-classpath") == 0
+			 || strcmp(argv[i], "-cp") == 0) {
+
+			i++;
+			if (argv[i] == 0) {
+				fprintf(stderr, 
+				    "Error: No path found for %s option.\n",
+				    argv[i - 1]);
+				exit(EXIT_FAILURE);
+			}
+
+			/* set the new classpath */
+			vmargs.classpath = strdup(argv[i]);
+		}
+		else if (strcmp(argv[i], "-addclasspath") == 0) {
 			char	*newcpath;
 			unsigned int      cpathlength;
 
@@ -491,7 +486,7 @@ options(char** argv, int argc)
 				fprintf(stderr, 
 				    "Error: No path found for %s option.\n",
 				    argv[i - 1]);
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
 
 			cpathlength = ((vmargs.classpath != NULL) ? strlen(vmargs.classpath) : 0)
@@ -502,7 +497,7 @@ options(char** argv, int argc)
 			/* Get longer buffer FIXME:  free the old one */
 			if ((newcpath = malloc(cpathlength)) == NULL) {
 				fprintf(stderr,  _("Error: out of memory.\n"));
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
 
 			/* Construct new classpath */
@@ -547,7 +542,7 @@ options(char** argv, int argc)
                         unsigned int      bootcpathlength;
 			const char *prefix =
 #if defined(ENABLE_BINRELOC)
-				LIBDIR
+				br_find_lib_dir(NULL)
 #else /* !defined(ENABLE_BINRELOC) */
 				DEFAULT_KAFFEHOME
 #endif /* defined(ENABLE_BINRELOC) */
@@ -565,7 +560,7 @@ options(char** argv, int argc)
                         /* Get longer buffer FIXME:  free the old one */
                         if ((newbootcpath = malloc(bootcpathlength)) == NULL) {
                                 fprintf(stderr,  _("Error: out of memory.\n"));
-                                exit(1);
+                                exit(EXIT_FAILURE);
                         }
 
                         /* Construct new boot classpath */
@@ -592,7 +587,7 @@ options(char** argv, int argc)
 			/* Get longer buffer FIXME:  free the old one */
 			if ((newbootcpath = malloc(bootcpathlength)) == NULL) {
 				fprintf(stderr,  _("Error: out of memory.\n"));
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
 
 			/* Construct new boot classpath */
@@ -617,7 +612,7 @@ options(char** argv, int argc)
 			/* Get longer buffer FIXME:  free the old one */
 			if ((newbootcpath = malloc(bootcpathlength)) == NULL) {
 				fprintf(stderr,  _("Error: out of memory.\n"));
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
 
 			/* Construct new boot classpath */
@@ -639,7 +634,7 @@ options(char** argv, int argc)
 			/* Get longer buffer FIXME:  free the old one */
 			if ((newbootcpath = malloc(bootcpathlength)) == NULL) {
 				fprintf(stderr,  _("Error: out of memory.\n"));
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
 
 			/* Construct new boot classpath */
@@ -654,7 +649,7 @@ options(char** argv, int argc)
 				i++;
 				if (argv[i] == 0) {
 					fprintf(stderr, _("Error: No stack size found for -ss option.\n"));
-					exit(1);
+					exit(EXIT_FAILURE);
 				}
 				sz = parseSize(argv[i]);
 			} else {
@@ -673,11 +668,17 @@ options(char** argv, int argc)
 				i++;
 				if (argv[i] == 0) {
 					fprintf(stderr,  _("Error: No heap size found for -mx option.\n"));
-					exit(1);
+					exit(EXIT_FAILURE);
 				}
-				vmargs.maxHeapSize = parseSize(argv[i]);
+				if (strcmp(argv[i], "unlimited") == 0)
+					vmargs.maxHeapSize = UNLIMITED_HEAP;
+				else
+					vmargs.maxHeapSize = parseSize(argv[i]);
 			} else {
-				vmargs.maxHeapSize = parseSize(&argv[i][j]);
+				if (strcmp(&argv[i][j], "unlimited") == 0)
+					vmargs.maxHeapSize = UNLIMITED_HEAP;
+				else
+					vmargs.maxHeapSize = parseSize(&argv[i][j]);
 			}
 		}
 		else if ((strncmp(argv[i], "-ms", (j=3)) == 0)
@@ -686,7 +687,7 @@ options(char** argv, int argc)
 				i++;
 				if (argv[i] == 0) {
 					fprintf(stderr,  _("Error: No heap size found for -ms option.\n"));
-					exit(1);
+					exit(EXIT_FAILURE);
 				}
 				vmargs.minHeapSize = parseSize(argv[i]);
 			} else {
@@ -698,7 +699,7 @@ options(char** argv, int argc)
 				i++;
 				if (argv[i] == 0) {
 					fprintf(stderr,  _("Error: No heap size found for -as option.\n"));
-					exit(1);
+					exit(EXIT_FAILURE);
 				}
 				vmargs.allocHeapSize = parseSize(argv[i]);
 			} else {
@@ -733,10 +734,52 @@ options(char** argv, int argc)
 			vmargs.enableVerboseClassloading = 1;
 		}
                 else if (strcmp(argv[i], "-jar") == 0) {
+			char	*newcpath;
+			unsigned int      cpathlength;
+
+			cpathlength = strlen(argv[i+1]) + 
+				strlen(path_separator) +
+				((vmargs.classpath!=NULL) ? strlen(vmargs.classpath) : 0) +
+				1;
+
+			newcpath = (char *)malloc (cpathlength);
+			if (newcpath == NULL) {
+				fprintf(stderr,  _("Error: out of memory.\n"));
+				exit(EXIT_FAILURE);
+			}
+
+			strcpy (newcpath, argv[i+1]);
+
+			if (vmargs.classpath != NULL) {
+				strcat (newcpath, path_separator);
+				strcat (newcpath, vmargs.classpath);
+				free ((void*)vmargs.classpath);
+			}
+
+			/* set the new classpath */
+			vmargs.classpath = newcpath;
+
                         isJar = 1;
                 }
 		else if (strncmp(argv[i], "-Xrun", 5) == 0) {
-			jvm_onload = argv[i];
+		  char *argPos;
+		  char *libName;
+		  int libnameLen;
+
+		  argPos = strchr(argv[i], ':');
+		  if (argPos != NULL)
+		    {
+		      libnameLen = argPos - &argv[i][5];
+		      vmargs.profilerArguments = strdup(argPos+1);
+		    }
+		  else
+		    libnameLen = strlen(argv[i]) - 1;
+
+		  libName = malloc(libnameLen+4);
+		  strcpy(libName, "lib");
+		  strncat(libName, &argv[i][5], libnameLen);
+
+		  vmargs.profilerLibname = libName;
 		}
 #if defined(KAFFE_PROFILER)
 		else if (strcmp(argv[i], "-prof") == 0) {
@@ -822,7 +865,7 @@ options(char** argv, int argc)
                                 fprintf(stderr, 
 					_("Error: -vmstats option requires a "
 					"second arg.\n"));
-                                exit(1);
+                                exit(EXIT_FAILURE);
                         }
                         statsSetMaskStr(argv[i]);
                 }
@@ -834,10 +877,10 @@ options(char** argv, int argc)
                                 fprintf(stderr, 
 					_("Error: -vmdebug option requires a "
 					"debug flag. Use `list' for a list.\n"));
-                                exit(1);
+                                exit(EXIT_FAILURE);
                         }
                         if (!dbgSetMaskStr(argv[i]))
-				exit(1);
+				exit(EXIT_FAILURE);
                 }
 #endif
                 else if (strcmp(argv[i], "-debug-fd") == 0) {
@@ -846,13 +889,13 @@ options(char** argv, int argc)
                         if (argv[i] == 0) { /* forgot second arg */
                                 fprintf(stderr, 
 					_("Error: -debug-fd an open descriptor.\n"));
-                                exit(1);
+                                exit(EXIT_FAILURE);
                         }
 			dbgSetDprintfFD(strtol(argv[i], &end, 10));
 			if (end != 0 && *end != '\0') {
 				fprintf(stderr,
 					_("Error: -debug-fd requires an integer.\n"));
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
                 }
 		else if (argv[i][1] ==  'D') {
@@ -988,6 +1031,8 @@ parseSize(char* arg)
 	case 'm': case 'M':
 		sz *= 1024 * 1024;
 		break;
+	default:
+		break;
 	}
 
 	return (sz);
