@@ -2,9 +2,12 @@
  * main.c
  * Generate native code stubs from .class files.
  *
- * Copyright (c) 1996, 1997, 2004
+ * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
+ * Copyright (c) 2004
+ *	Kaffe.org contributors. See ChangeLog for details. All rights reserved.
+ *   
  * See the file "license.terms" for information on usage and redistribution 
  * of this file. 
  */
@@ -15,6 +18,21 @@
 #include "kaffeh-support.h"
 #include "utf8const.h"
 #include "debug.h"
+#include "system.h"
+
+#if defined(HAVE_GETTEXT)
+#include <libintl.h>
+#define _(T) gettext(T)
+#else
+#define _(T) (T)
+#endif
+#if defined(HAVE_LC_MESSAGES)
+#include <locale.h>
+#endif
+
+#ifdef ENABLE_BINRELOC
+#include "binreloc.h"
+#endif
 
 #define	BUFSZ	1024
 #define	PATHSZ	1024
@@ -60,9 +78,18 @@ int
 main(int argc, char* argv[])
 {
 	char* nm;
-	int i, first = 1;
+	int i, j, first = 1;
 	int farg;
 
+#if defined(ENABLE_BINRELOC)
+	BrInitError error;
+
+	if( br_init(&error) == 0 && error != BR_INIT_ERROR_DISABLED) {
+        	printf ("Warning: BinReloc failed to initialize (error code %d)\n", error);
+        	printf ("Will fallback to hardcoded default path.\n");
+    	};
+#endif
+
 	/* Process arguments */
 	farg = options(argc, argv);
 
@@ -88,28 +115,37 @@ main(int argc, char* argv[])
 	for (nm = argv[farg]; nm != 0; nm = argv[++farg]) {
 
 		/* Derive various names from class name */
-		for (i = 0; nm[i] != 0; i++) {
-			if (i >= BUFSZ - 100) {
+		for (i = j = 0; nm[i] != 0; i++, j++) {
+			if (i >= BUFSZ - 100 || j >= BUFSZ - 105) {
 				dprintf(
 				    "kaffeh: class name too long\n");
 				exit(1);
 			}
 			switch (nm[i]) {
 			case '/':
-			case '$':
 			case '.':
-				className[i] = '_';
+				className[j] = '_';
 				pathName[i] = '/';
 				includeName[i] = '_';
 				break;
+			case '$':
+				className[j++] = '_';
+				className[j++] = '0';
+				className[j++] = '0';
+				className[j++] = '0';
+				className[j++] = '2';
+				className[j] = '4';
+				pathName[i] = nm[i];
+				includeName[i] = '_';
+				break;
 			default:
-				className[i] = nm[i];
+				className[j] = nm[i];
 				pathName[i] = nm[i];
 				includeName[i] = nm[i];
 				break;
 			}
 		}
-		className[i] = 0;
+		className[j] = 0;
 		pathName[i] = 0;
 		includeName[i] = 0;
 
@@ -202,6 +238,26 @@ options(int argc, char** argv)
 {
 	int i;
 
+
+	char * bootclasspath = NULL;
+	char * classpath = NULL;
+
+	bootclasspath = malloc(1);
+	if (bootclasspath == NULL) 
+	  {
+	    fprintf(stderr,  _("Error: out of memory.\n"));
+	    exit(EXIT_FAILURE);
+	  }
+	*bootclasspath = '\0';
+
+	classpath = malloc(1);
+	if (classpath == NULL) 
+	  {
+	    fprintf(stderr,  _("Error: out of memory.\n"));
+	    exit(EXIT_FAILURE);
+	  }
+	*classpath = '\0';
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] != '-') {
 			break;
@@ -231,8 +287,54 @@ options(int argc, char** argv)
 		}
 #endif /*KAFFE_VMDEBUG*/
 		else if (strcmp(argv[i], "-classpath") == 0) {
-			i++;
-			strcpy(realClassPath, argv[i]);
+		  char * newcp;
+		  size_t cplen;
+
+		  i++;
+
+		  cplen = strlen(classpath)
+		    + strlen(argv[i])
+		    + strlen(path_separator)
+		    + 1;
+
+		  newcp = malloc(cplen);
+		  if (newcp == NULL)
+		    {
+		      fprintf(stderr,  _("Error: out of memory.\n"));
+		      exit(EXIT_FAILURE);
+		    }
+		  
+		  strcpy(newcp, classpath);
+		  strcat(newcp, path_separator);
+		  strcat(newcp, argv[i]);
+
+		  free(classpath);
+		  classpath = newcp;
+		}
+		else if (strcmp(argv[i], "-bootclasspath") == 0) {
+		  char * newcp;
+		  size_t cplen;
+
+		  i++;
+
+		  cplen = strlen(bootclasspath)
+		    + strlen(argv[i])
+		    + strlen(path_separator)
+		    + 1;
+
+		  newcp = malloc(cplen);
+		  if (newcp == NULL)
+		    {
+		      fprintf(stderr,  _("Error: out of memory.\n"));
+		      exit(EXIT_FAILURE);
+		    }
+		  
+		  strcpy(newcp, bootclasspath);
+		  strcat(newcp, path_separator);
+		  strcat(newcp, argv[i]);
+
+		  free(bootclasspath);
+		  bootclasspath = newcp;
 		}
 		else if (strcmp(argv[i], "-o") == 0) {
 			i++;
@@ -247,6 +349,34 @@ options(int argc, char** argv)
 		}
 	}
 
+
+	if (strlen(bootclasspath) > 0) 
+	  {
+	    strcpy(realClassPath, bootclasspath);
+	  } 
+	else 
+	  {
+
+#if defined(ENABLE_BINRELOC)
+            char *prefixPath = br_find_prefix(NULL);
+	    char *rtJarPath = br_build_path(prefixPath, "jre/lib/rt.jar");
+
+	    strcpy(realClassPath, rtJarPath);
+
+	    free(rtJarPath);
+	    free(prefixPath);
+#endif
+	  }
+
+	free(bootclasspath);
+
+	if (strlen(classpath) > 0)
+	  {
+	    strcat(realClassPath, classpath);
+	  }
+
+	free(classpath);
+
 	/* Return first no-flag argument */
 	return (i);
 }
@@ -263,6 +393,7 @@ usage(void)
 	dprintf("	-help			Print this message\n");
 	dprintf("	-version		Print version number\n");
 	dprintf("	-classpath <path>	Set classpath\n");
+	dprintf("	-bootclasspath <path>	Set classpath\n");
 	dprintf("	-jni			Generate JNI interface\n");
 #ifdef KAFFE_VMDEBUG
 	dprintf("	-Xdebug <opts>		Kaffe debug options.\n");
