@@ -42,7 +42,6 @@
 #include "md.h"
 #include "jni.h"
 #include "soft.h"
-#include "methodCache.h"
 #include "gcj/gcj.h"
 #include "xprofiler.h"
 #include "debugFile.h"
@@ -75,8 +74,46 @@ static bool resolveObjectFields(Hjava_lang_Class*, errorInfo *einfo);
 static bool resolveStaticFields(Hjava_lang_Class*, errorInfo *einfo);
 static bool resolveConstants(Hjava_lang_Class*, errorInfo *einfo);
 static bool resolveInterfaces(Hjava_lang_Class *class, errorInfo *einfo);
+static parsed_signature_t *duplicateParsedSignature (parsed_signature_t *, errorInfo *);
+
+static struct Hjava_security_ProtectionDomain  *defaultProtectionDomain;
 
 
+void
+initialiseSecurity (void)
+{
+	struct Hjava_lang_Object *codeSource;
+	struct Hjava_lang_Object *permissions;
+	struct Hjava_lang_Object *allPermission;
+
+	codeSource = execute_java_constructor ("java/security/CodeSource",
+					       NULL,
+					       NULL,
+					       "(Ljava/net/URL;[Ljava/security/cert/Certificate;)V",
+					       NULL,
+					       NULL);
+
+	permissions = execute_java_constructor ("java/security/Permissions",
+						NULL,
+						NULL,
+						"()V");
+
+	allPermission = execute_java_constructor ("java/security/AllPermission",
+						  NULL,
+						  NULL,
+						  "()V");
+	
+	do_execute_java_method(NULL, permissions, "add", "(Ljava/security/Permission;)V",
+			       NULL, 0, allPermission);
+	do_execute_java_method(NULL, permissions, "setReadOnly", "()V", NULL, 0);
+
+	defaultProtectionDomain = (struct Hjava_security_ProtectionDomain *)execute_java_constructor ("java/security/ProtectionDomain",
+							    NULL,
+							    NULL,
+							    "(Ljava/security/CodeSource;Ljava/security/PermissionCollection;)V",
+							    codeSource,
+							    permissions);
+}
 
 #if !defined(ALIGNMENT_OF_SIZE)
 #define	ALIGNMENT_OF_SIZE(S)	(S)
@@ -203,7 +240,9 @@ retry:
 				success = false;
 				goto done;
 			}
-			KGC_addWeakRef(main_collector, class->superclass, &(class->superclass));
+			KGC_addWeakRef(main_collector, 
+				       class->superclass, 
+				       (void **)&(class->superclass));
 			if( !(class->accflags & ACC_INTERFACE) &&
 			    (class->superclass->accflags & ACC_INTERFACE)) {
 				postExceptionMessage(
@@ -666,15 +705,28 @@ expandMethods(Hjava_lang_Class *cl, Method *imeth, errorInfo *einfo)
 		int i;
 		
 		i = CLASS_NMETHODS(cl);
-		CLASS_NMETHODS(cl) = i + 1;
 		CLASS_METHODS(cl) = new_methods;
 		utf8ConstAddRef(imeth->name);
-		utf8ConstAddRef(imeth->parsed_sig->signature);
 		new_methods[i] = *imeth;
+
+		/* Allocate a new parsed_signature_t for the method. We can't use the one
+		 * of the implemented method as destroyClass would then try to free it
+		 * twice.
+		 */
+		new_methods[i].parsed_sig = duplicateParsedSignature (imeth->parsed_sig, einfo);
+		if (new_methods[i].parsed_sig == NULL)
+		{
+			gc_free (new_methods);
+			return 0;
+		}
+
 		new_methods[i].ndeclared_exceptions = -1;
 		new_methods[i].declared_exceptions_u.remote_exceptions =
 			imeth;
 		new_methods[i].class = cl;
+
+		CLASS_NMETHODS(cl) = i + 1;
+
 		retval = 1;
 	}
 	else
@@ -837,7 +889,14 @@ resolveInterfaces(Hjava_lang_Class *class, errorInfo *einfo)
 				newifaces[i] = nclass->interfaces[j];
 			}
 		}
+		if (class->interfaces != NULL)
+		  gc_rm_ref(class->interfaces);
 		class->interfaces = newifaces;
+		if (!gc_add_ref(class->interfaces)) {
+			postOutOfMemory(einfo);
+			success = false;
+			goto done;
+		}	
 	}
 
 	/* don't set total_interface_len before interfaces to avoid
@@ -940,6 +999,13 @@ internalSetupClass(Hjava_lang_Class* cl, Utf8Const* name, int flags,
 	cl->inner_classes = NULL;
 	cl->nr_inner_classes = 0;
 	cl->this_inner_index = -1;
+
+	/* set protection domain to the default */
+	if (loader == NULL)
+	{
+		cl->protectionDomain = defaultProtectionDomain;
+	}
+
 	return 1;
 }
 
@@ -1046,7 +1112,6 @@ addInnerClasses(Hjava_lang_Class* c, size_t len UNUSED, classFile* fp,
 		readu2(&ic->inner_class_accflags, fp);
 
 		if (c->this_index && ic->inner_class == c->this_index) {
-		    c->accflags = (c->accflags & ~ACC_MASK) | (ic->inner_class_accflags & ACC_MASK);
 		    c->this_inner_index = i;
 		}
 	}
@@ -1557,7 +1622,7 @@ DBG(VMCLASSLOADER,
 );
 
 		clazz = findClass(centry, &info);
-		if (clazz == 0) {
+		if (clazz == NULL) {
 			goto bad;
 		}
 		/* we won't ever want to lose these classes */
@@ -1569,10 +1634,9 @@ DBG(VMCLASSLOADER,
 		(*class) = centry->data.cl = clazz;
 	}
 	unlockStaticMutex(&centry->slock);
-	
 	if (!(*class))
 		(*class) = centry->data.cl;
-	
+
 	if (processClass(centry->data.cl, CSTATE_LINKED, &info) == true) {
 		assert(centry->state == NMS_DONE);
 		return;
@@ -1581,7 +1645,7 @@ DBG(VMCLASSLOADER,
 bad:
 	dprintf("Couldn't find or load essential class `%s' %s %s\n",
 			name, info.classname, (char*)info.mess);
-	ABORT();
+	KAFFEVM_ABORT();
 }
 
 /*
@@ -1663,7 +1727,7 @@ resolveObjectFields(Hjava_lang_Class* class, errorInfo *einfo)
 	int maxalign;
 	int oldoffset;
 	int *map;
-	jbool is_reference;
+	jboolean is_reference;
 
 	/* Find start of new fields in this object.  If start is zero, we must
 	 * allow for the object headers.
@@ -1868,7 +1932,7 @@ resolveStaticFields(Hjava_lang_Class* class, errorInfo *einfo)
 	constants* pool;
 	Utf8Const* utf8;
 	Field* fld;
-	int idx;
+	unsigned int idx;
 	int n;
 
 	/* No locking here, assume class is already locked. */
@@ -1929,6 +1993,8 @@ resolveStaticFields(Hjava_lang_Class* class, errorInfo *einfo)
 				*(jref*)mem = (jref)CLASS_CONST_DATA(class, idx);
 				FIELD_SIZE(fld) = PTR_TYPE_SIZE;
 				break;
+			default:
+				break;
 			}
 		}
 	}
@@ -1951,6 +2017,14 @@ getInheritedMethodIndex(Hjava_lang_Class *super, Method *meth)
 		int j = CLASS_NMETHODS(super);
 		Method* mt = CLASS_METHODS(super);
 		for (; --j >= 0;  ++mt) {
+			/* skip methods that are private or static */
+			if ((mt->accflags & (ACC_PRIVATE|ACC_STATIC)) != 0)
+				continue;
+
+			/* skip inaccessible methods */
+			if (!checkAccess (meth->class, super, mt->accflags))
+				continue;
+
 			if (utf8ConstEqual (mt->name, meth->name) &&
 			    utf8ConstEqual (METHOD_SIG(mt), METHOD_SIG(meth)))
 			{
@@ -2139,6 +2213,10 @@ buildInterfaceDispatchTable(Hjava_lang_Class* class, errorInfo *einfo)
 		postOutOfMemory(einfo);
 		return (false);
 	}
+	if (!gc_add_ref(class->itable2dtable)) {
+		postOutOfMemory(einfo);
+		return false;
+	}
 	class->itable2dtable[0] = class;
 	j = 1;
 	for (i = 0; i < class->total_interface_len; i++) {
@@ -2223,7 +2301,8 @@ buildInterfaceDispatchTable(Hjava_lang_Class* class, errorInfo *einfo)
 static bool
 computeInterfaceImplementationIndex(Hjava_lang_Class* clazz, errorInfo* einfo)
 {
-	int i, j, k;
+	int j, k;
+	unsigned int i;
 	int found_i;
 	bool rc = false;
 	Hjava_lang_Class** ifcs;
@@ -2614,6 +2693,8 @@ sizeofSigChar(char ch, bool want_wide_refs)
 	case '[':
 	case 'L':
 		return want_wide_refs ? sizeof(void*) / sizeof(int32) : 1;
+	default:
+		break;
 	}
 
 	return -1;
@@ -2640,7 +2721,7 @@ sizeofSigItem(const char** strp, bool want_wide_refs)
 				break;
 			default:
 				count = -1;	/* avoid compiler warning */
-				ABORT();
+				KAFFEVM_ABORT();
 			}
 		} else {
 			while (*str == '[')
@@ -2705,7 +2786,7 @@ sizeofSigMethod(Method *meth, bool want_wide_refs)
 	return (size);
 }
 
-/*
+/**
  * Count the number of arguments in a signature.
  */
 int
@@ -2720,6 +2801,31 @@ countArgsInSignature(const char *signature)
 	return (nargs);
 }
 
+/**
+ * Duplicates a parsed signature.
+ */
+static parsed_signature_t*
+duplicateParsedSignature(parsed_signature_t *orig, errorInfo *einfo)
+{
+	parsed_signature_t *ret;
+	size_t sizeOfSignature;
+
+	sizeOfSignature = sizeof(*ret) + orig->nargs * sizeof(ret->ret_and_args[0]);
+
+	ret = (parsed_signature_t *)gc_malloc (sizeOfSignature, KGC_ALLOC_CLASSMISC);
+
+	if (ret == NULL) {
+		postOutOfMemory(einfo);
+		return NULL;
+	}
+
+	memcpy (ret, orig, sizeOfSignature);
+
+	utf8ConstAddRef (PSIG_UTF8(ret));
+
+	return ret;
+}
+
 /*
  * Parse a method signature and return an array of pointers to classes.
  */
@@ -2749,6 +2855,8 @@ parseSignature(Utf8Const *signature, errorInfo *einfo)
 	++sig_iter; /* skip `)' */
 	PSIG_RET(sig) = sig_iter - signature->data;
 
+	PSIG_RNARGS(sig) = KaffeVM_countRealNumberOfArgs(sig); 
+
 	return sig;
 }
 
@@ -2847,6 +2955,10 @@ lookupArray(Hjava_lang_Class* c, errorInfo *einfo)
 		arr_flags |= ACC_PUBLIC;
 	}
 	internalSetupClass(arr_class, arr_name, arr_flags, 0, 0, c->loader, NULL);
+
+	/* use protection domain of element class for array classes */
+	arr_class->protectionDomain = c->protectionDomain;
+
 	arr_class->superclass = ObjectClass;
 	if (buildDispatchTable(arr_class, einfo) == false) {
 		centry->data.cl = c = NULL;
@@ -2864,7 +2976,7 @@ lookupArray(Hjava_lang_Class* c, errorInfo *einfo)
 	addInterfaces(arr_class, 2, arr_interfaces);
 
 	arr_class->total_interface_len = arr_class->interface_len;
-	arr_class->head.vtable = getClassClass()->vtable;
+	arr_class->head.vtable = getClassVtable();
 	arr_class->state = CSTATE_COMPLETE;
 	arr_class->centry = centry;
 
