@@ -2,10 +2,12 @@
  * exception.c
  * Handle exceptions for the interpreter or translator.
  *
- * Copyright (c) 1996, 1997, 2004
+ * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
+ *
  * Copyright (c) 2003
  * 	Mark J. Wielaard <mark@klomp.org>
+ *
  * Copyright (c) 2004
  *      Kaffe.org contributors. See ChangeLogs for details. All rights reserved.
  *
@@ -26,7 +28,7 @@
 #include "config-mem.h"
 #include "config-setjmp.h"
 #include "config-hacks.h"
-#include "kaffe/jni_md.h"
+#include "jni_md.h"
 #include "gtypes.h"
 #include "access.h"
 #include "object.h"
@@ -324,8 +326,8 @@ throwException(struct Hjava_lang_Throwable* eobj)
 
 	if (eobj == 0) {
 		dprintf("Exception thrown on null object ... aborting\n");
-		ABORT();
-		EXIT(1);
+		KAFFEVM_ABORT();
+		KAFFEVM_EXIT(1);
 	}
 	vmstate = unhand(eobj)->vmState;
 	if (vmstate == 0) {
@@ -334,7 +336,7 @@ throwException(struct Hjava_lang_Throwable* eobj)
 		unhand(eobj)->vmState = vmstate;
 	}
 	backtrace = buildStackTrace(NULL);
-	unhand(vmstate)->backtrace = backtrace;
+	unhand(vmstate)->vmdata = backtrace;
 	dispatchException(eobj, (stackTraceInfo*)backtrace);
 }
 
@@ -349,8 +351,8 @@ throwExternalException(Hjava_lang_Throwable* eobj)
 {
 	if (eobj == 0) {
 		dprintf("Exception thrown on null object ... aborting\n");
-		ABORT();
-		EXIT(1);
+		KAFFEVM_ABORT();
+		KAFFEVM_EXIT(1);
 	}
 	dispatchException(eobj, (stackTraceInfo*)buildStackTrace(NULL));
 }
@@ -366,7 +368,7 @@ throwOutOfMemory(void)
 		throwException(err);
 	}
 	dprintf("(Insufficient memory)\n");
-	EXIT(-1);
+	KAFFEVM_EXIT(-1);
 }
 #endif
 
@@ -409,14 +411,14 @@ dispatchException(Hjava_lang_Throwable* eobj, stackTraceInfo* baseFrame)
 	     lastJniFrame = lastJniFrame->prev);
 
 	DBG(ELOOKUP,
-		dprintf ("dispatchException(): lastJniFrame is %p, fp 0x%x\n", lastJniFrame, (lastJniFrame?lastJniFrame->frame.jni.fp:0)); );
+		dprintf ("dispatchException(): lastJniFrame is %p, fp %p\n", lastJniFrame, (lastJniFrame?lastJniFrame->frame.jni.fp:0)); );
 
 	/*
 	 * now walk up the stack 
 	 */
 	for (frame = baseFrame; frame->meth != ENDOFSTACK; frame++) {
 		bool 			foundHandler;
-		uintp 			handler;
+		uintp 			handler = 0;
 		Hjava_lang_Object*	obj;
 
 		/*
@@ -516,7 +518,7 @@ unhandledException(Hjava_lang_Throwable *eobj)
 		}
 	}
 	printStackTrace((Hjava_lang_Throwable*)eobj, NULL, 1);
-	ABORT();
+	KAFFEVM_ABORT();
 }
 
 /*
@@ -545,7 +547,7 @@ stackOverflowException(struct _exceptionFrame *frame)
 	soe = (Hjava_lang_Throwable*)newObject(javaLangStackOverflowError);
 	vmstate = (Hjava_lang_VMThrowable*)newObject(javaLangVMThrowable);
 	backtrace = buildStackTrace(frame);
-	unhand(vmstate)->backtrace = backtrace;
+	unhand(vmstate)->vmdata = backtrace;
 	unhand(soe)->vmState = vmstate;
 #if defined(HAVE_GCJ_SUPPORT)
 	FAKE_THROW_FRAME();
@@ -566,8 +568,9 @@ nullException(struct _exceptionFrame *frame)
 	npe = (Hjava_lang_Throwable*)newObject(javaLangNullPointerException);
 	vmstate = (Hjava_lang_VMThrowable*)newObject(javaLangVMThrowable);
 	backtrace = buildStackTrace(frame);
-	unhand(vmstate)->backtrace = backtrace;
+	unhand(vmstate)->vmdata = backtrace;
 	unhand(npe)->vmState = vmstate;
+	unhand(npe)->cause = npe;
 #if defined(HAVE_GCJ_SUPPORT)
 	FAKE_THROW_FRAME();
 #endif /* defined(HAVE_GCJ_SUPPORT) */
@@ -587,7 +590,7 @@ floatingException(struct _exceptionFrame *frame)
 	ae = (Hjava_lang_Throwable*)newObject(javaLangArithmeticException);
 	vmstate = (Hjava_lang_VMThrowable*)newObject(javaLangVMThrowable);
 	backtrace = buildStackTrace(frame);
-	unhand(vmstate)->backtrace = backtrace;
+	unhand(vmstate)->vmdata = backtrace;
 	unhand(ae)->vmState = vmstate;
 #if defined(HAVE_GCJ_SUPPORT)
 	FAKE_THROW_FRAME();
@@ -630,8 +633,8 @@ DBG(ELOOKUP,
 		uintp end_pc = eptr[i].end_pc;
 		uintp handler_pc = eptr[i].handler_pc;
 
-DBG(ELOOKUP,	dprintf("  Handler %d covers %#lx-%#lx\n", i, 
-			(long) start_pc, (long) end_pc); );
+DBG(ELOOKUP,	dprintf("  Handler %d covers %#lx-%#lx catches %s\n", i, 
+			(long) start_pc, (long) end_pc, eptr[i].catch_type==NULL?"all":CLASS_CNAME(eptr[i].catch_type)););
 		if (_pc < start_pc || _pc >= end_pc) {
 			continue;
 		}
