@@ -36,7 +36,9 @@
 #include "access.h"
 #include "gcj/gcj.h"
 #include "defs.h"
-#include "prefix.h"
+#if defined(ENABLE_BINRELOC)
+#include "binreloc.h"
+#endif
 
 #ifdef __riscos__
 #include <unixlib/local.h>
@@ -46,7 +48,7 @@
    in a ClassLoader.  */
 #define HANDLE_MANIFEST_CLASSPATH	1
 
-#define KLASSES_JAR	"Klasses.jar"
+#define KLASSES_JAR	"rt.jar"
 
 classpathEntry* classpath;
 
@@ -66,7 +68,7 @@ static int insertClasspath(const char* cp, int prepend);
 
 #if defined(HANDLE_MANIFEST_CLASSPATH)
 static int isEntryInClasspath(const char*);
-static uint8* getManifestMainAttribute(jarFile*, const char*);
+static char* getManifestMainAttribute(jarFile*, const char*);
 static void handleManifestClassPath (classpathEntry *);
 #endif
 
@@ -155,6 +157,9 @@ DBG(CLASSLOOKUP,
 		}
 		return (class);
 
+	case CP_INVALID:
+	case CP_SOFILE:
+	case CP_BYTEARRAY:
 	default:
 		break;
 	}
@@ -167,7 +172,7 @@ DBG(CLASSLOOKUP,
 	if (strcmp(cname, "java/lang/ClassNotFoundException") == 0 ||
 	    strcmp(cname, "java/lang/Object") == 0) {
 		dprintf("Cannot find essential class '%s' in class library ... aborting.\n", cname);
-		ABORT();
+		KAFFEVM_ABORT();
 	}
 	return (NULL);
 }
@@ -204,7 +209,7 @@ DBG(CLASSLOOKUP,dprintf("Processing classpath entry '%s'\n", ptr->path); );
 		case CP_ZIPFILE:
 		{
 			jarEntry* entry;
-			char* data;
+			unsigned char* data;
 
 DBG(CLASSLOOKUP,	dprintf("Opening JAR file %s for %s\n", ptr->path, cname); );
 			if (ptr->u.jar == 0) {
@@ -250,8 +255,8 @@ DBG(CLASSLOOKUP,	dprintf("Opening JAR file %s for %s\n", ptr->path, cname); );
 		case CP_DIR:
 		{
 			struct stat sbuf;
-			char* data;
-			
+			unsigned char* data;
+
 			buf = checkPtr(KMALLOC(strlen(ptr->path)
 			    + strlen(file_separator) + strlen(cname) + 1));
 			sprintf(buf, "%s%s%s",
@@ -470,10 +475,10 @@ discoverClasspath(const char* home)
 		return;
 	}
 
-	/* Add '.' and <home>/Klasses.jar at the beginning */
+	/* Add '.' and <home>/lib/rt.jar at the beginning */
 	addClasspath(".");
-	buf = KMALLOC(strlen(home) + strlen(KLASSES_JAR) + 2);
-	sprintf(buf, "%s/%s", home, KLASSES_JAR);
+	buf = KMALLOC(strlen(home) + strlen(KLASSES_JAR) + 6);
+	sprintf(buf, "%s/lib/%s", home, KLASSES_JAR);
 	addClasspath(buf);
 	KFREE(buf);
 
@@ -619,13 +624,13 @@ isEntryInClasspath(const char *path)
 }
 
 
-static uint8*
+static char*
 getManifestMainAttribute(jarFile* file, const char* attrName)
 {
 	jarEntry* mf;
-	uint8* mfdata;
-	uint8* attrEntry;
-	uint8* ret;
+	char* mfdata;
+	char* attrEntry;
+	char* ret;
 	size_t i;
 	int posAttrValue;
 
@@ -635,7 +640,7 @@ getManifestMainAttribute(jarFile* file, const char* attrName)
 		return (NULL);
 
 	/* Read it */
-	mfdata = getDataJarFile(file, mf);
+	mfdata = (char*)getDataJarFile(file, mf);
 	if (mfdata == 0)
 		return (NULL);
 
@@ -791,8 +796,9 @@ discoverClassHome(void)
 
       char* p;
       const char* referenceName = "rt.jar";
+      char *exeFilename = br_find_exe(NULL);
 
-      strcpy(discoveredClassHome, SELFPATH);
+      strcpy(discoveredClassHome, exeFilename);
 
       while ((p = strrchr(discoveredClassHome, file_separator[0]))) {
          if (p + 1 + strlen(referenceName) 
@@ -800,16 +806,19 @@ discoverClassHome(void)
             strcpy(p + 1, referenceName);
             if (0 == access(discoveredClassHome, R_OK)) {
                *p = '\0';                    
+	       free(exeFilename);
                return discoveredClassHome;
             }
          }
          *p = '\0';                    
       }
+      free(exeFilename);
    }
    else {
       fprintf(stderr, "WARNING: file_separator not a single character, unable to discover lib directory\n");
    }
 #endif
+
    
    return NULL;
 }
