@@ -12,9 +12,14 @@
 #ifndef __icode_h
 #define	__icode_h
 
+#include "config.h"
+#include "config-int.h"
 #include "slots.h"
 #include "soft.h"
 
+#define  JINT_MIN        INT32_MIN
+#define  JLONG_MIN       INT64_MIN
+
 #if defined(KAFFE_VMDEBUG)
 static const int32 UNINITIALIZED_STACK_SLOT = 0x00c0ffee;
 #endif
@@ -129,6 +134,7 @@ static const int32 UNINITIALIZED_STACK_SLOT = 0x00c0ffee;
 						virtualMachine(m, sp+1, retval, thread_data)
 
 #define	ret()					goto end
+#define exit_method()				do { } while(0)
 
 #define	returnarg_int(s)			retval[0].v.tint = (s)[0].v.tint
 #define	returnarg_ref(s)			retval[0].v.taddr = (s)[0].v.taddr
@@ -178,7 +184,7 @@ static const int32 UNINITIALIZED_STACK_SLOT = 0x00c0ffee;
 						 ((uint8*)(f)->v.taddr)[3])
 #define	load_code_ref				load_key
 
-#define	softcall_breakpoint()			ABORT()
+#define	softcall_breakpoint()			KAFFEVM_ABORT()
 
 #define	move_double(t, f)			(t)[0].v.tdouble = (f)[0].v.tdouble
 #define	move_double_const(t, c)			(t)[0].v.tdouble = (c)
