@@ -2,10 +2,10 @@
  * jni.c
  * Java Native Interface.
  *
- * Copyright (c) 1996, 1997, 2004
+ * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * Copyright (c) 2004
+ * Copyright (c) 2004-2005
  * 	The Kaffe.org's developers. See ChangeLog for details.
  *
  * See the file "license.terms" for information on usage and redistribution 
@@ -41,6 +41,7 @@
 #include "jni_funcs.h"
 #include "native-wrapper.h"
 #include "kaffe_jni.h"
+#include "stackTrace.h"
 
 extern struct JNINativeInterface Kaffe_JNINativeInterface;
 extern KaffeVM_Arguments Kaffe_JavaVMInitArgs;
@@ -56,48 +57,6 @@ static jint Kaffe_GetVersion(JNIEnv*);
 static jclass Kaffe_FindClass(JNIEnv*, const char*);
 static jint Kaffe_ThrowNew(JNIEnv*, jclass, const char*);
 static jint Kaffe_Throw(JNIEnv* env, jobject obj);
-static void NONRETURNING Kaffe_FatalError(JNIEnv* env UNUSED, const char* mess);
-
-void
-KaffeJNI_addJNIref(jref obj)
-{
-	jnirefs* table;
-	int idx;
-
-	table = THREAD_DATA()->jnireferences;
-
-	if (table->used == table->frameSize) {
-	  Kaffe_FatalError(THREAD_JNIENV(), "No more room for local references");
-	}
-
-	idx = table->next;
-	for (;;) {
-		if (table->objects[idx] == 0) {
-			table->objects[idx] = obj;
-			table->used++;
-			table->next = (idx + 1) % table->frameSize;
-			return;
-		}
-		idx = (idx + 1) % table->frameSize;
-	}
-}
-
-void
-KaffeJNI_removeJNIref(jref obj)
-{
-	int idx;
-	jnirefs* table;
-
-	table = THREAD_DATA()->jnireferences;
-
-	for (idx = 0; idx < table->frameSize; idx++) {
-		if (table->objects[idx] == obj) {
-			table->objects[idx] = 0;
-			table->used--;
-			return;
-		}
-	}
-}
 
 /*
  * Everything from this point to Kaffe_GetVersion is not
@@ -107,48 +66,15 @@ KaffeJNI_removeJNIref(jref obj)
  * Everything from Kaffe_GetVersion to Kaffe_GetJavaVM
  * should be bracketed with BEGIN and END _EXCEPTION_HANDLING.
  */
-static void NONRETURNING
-Kaffe_FatalError(JNIEnv* env UNUSED, const char* mess)
+void NONRETURNING
+KaffeJNI_FatalError(JNIEnv* env UNUSED, const char* mess)
 {
 	kprintf(stderr, "FATAL ERROR: %s\n", mess);
-	exit(-1);
+	abort();
 }
 
-static void
-Kaffe_DeleteGlobalRef(JNIEnv* env UNUSED, jref obj)
-{
-#if defined(ENABLE_JVMPI)
-	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_JNI_GLOBALREF_FREE) )
-	{
-		JVMPI_Event ev;
-
-		ev.event_type = JVMPI_EVENT_JNI_GLOBALREF_FREE;
-		ev.u.jni_globalref_free.ref_id = obj;
-		jvmpiPostEvent(&ev);
-	}
-#endif
-	gc_rm_ref(obj);
-}
-
-static void
-/* ARGSUSED */
-Kaffe_DeleteLocalRef(JNIEnv* env UNUSED, jref obj)
-{
-	REMOVE_REF(obj);
-}
-
-static jboolean
-Kaffe_IsSameObject(JNIEnv* env UNUSED, jobject obj1, jobject obj2)
-{
-	if (obj1 == obj2) {
-		return (JNI_TRUE);
-	}
-	else {
-		return (JNI_FALSE);
-	}
-}
 static jint
-Kaffe_GetVersion(JNIEnv* UNUSED env)
+Kaffe_GetVersion(JNIEnv* env UNUSED)
 {
        return Kaffe_JavaVMArgs.version;
 }
@@ -159,154 +85,30 @@ Kaffe_GetVersion(JNIEnv* UNUSED env)
 static void
 postError(JNIEnv* env, errorInfo* info)
 {
-	Kaffe_Throw(env, error2Throwable(info));
-}
-
-static jref
-Kaffe_NewGlobalRef(JNIEnv* env, jref obj)
-{
-	BEGIN_EXCEPTION_HANDLING(NULL);
-	if (!gc_add_ref(obj)) {
-		errorInfo info;
-		postOutOfMemory(&info);
-		postError(env, &info);
-	}
-#if defined(ENABLE_JVMPI)
-	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_JNI_GLOBALREF_ALLOC) )
-	{
-		JVMPI_Event ev;
-
-		ev.event_type = JVMPI_EVENT_JNI_GLOBALREF_ALLOC;
-		ev.u.jni_globalref_alloc.obj_id = obj;
-		ev.u.jni_globalref_alloc.ref_id = obj;
-		jvmpiPostEvent(&ev);
-	}
-#endif
-	END_EXCEPTION_HANDLING();
-	return obj;
-}
-
-static jint
-KaffeJNI_PushLocalFrame(JNIEnv* env UNUSED, jint capacity)
-{
-  jnirefs *table;
-
-  BEGIN_EXCEPTION_HANDLING(-1);
-
-  if (capacity <= 0)
-    return -1;  
-
-  table = gc_malloc
-    (sizeof(jnirefs) + sizeof(jref)*capacity,
-     KGC_ALLOC_STATIC_THREADDATA);
-  if (table == NULL)
-    {
-      errorInfo info;
-      postOutOfMemory(&info);
-      postError(env, &info);
-      return -1;
-    }
-
-  table->prev = thread_data->jnireferences;
-  table->frameSize = capacity;
-  table->localFrames = thread_data->jnireferences->localFrames+1;
-
-  thread_data->jnireferences = table;
-
-  END_EXCEPTION_HANDLING();
-  
-  return 0;
-}
-
-static jint
-KaffeJNI_EnsureLocalCapacity(JNIEnv* env, jint capacity)
-{
-  jint ret;
-
-  BEGIN_EXCEPTION_HANDLING(-1);
-
-  if (thread_data->jnireferences->used+capacity > 
-      thread_data->jnireferences->frameSize)
-    ret = KaffeJNI_PushLocalFrame(env, capacity);
-
-  END_EXCEPTION_HANDLING();
-
-  return ret;
-}
-
-static jobject
-KaffeJNI_PopLocalFrame(JNIEnv* env UNUSED, jobject obj)
-{
-  int localFrames;
-  int i;
-  jnirefs *table;
-
-  BEGIN_EXCEPTION_HANDLING(NULL);
-
-  table = thread_data->jnireferences;
-  localFrames = table->localFrames;
-
-  /* We must not delete the top JNI local frame as it is done by
-   * the native wrapper.
-   */
-  if (localFrames == 1)
-    goto popframe_end;
-  
-  localFrames = table->localFrames;
-  for (localFrames = table->localFrames; localFrames >= 1; localFrames--)
-    {
-      thread_data->jnireferences = table->prev;
-      gc_free(table);
-      table = thread_data->jnireferences;
-    }
-  
-  if (obj != NULL)
-    {
-      for (i = 0; i < table->frameSize; i++)
-	if (table->objects[i] == obj)
-	  break;
-      
-      /* If the object is not already referenced, add a new reference to it.
-       */
-      if (i == table->frameSize)
-	ADD_REF(obj); 
-    }
-  
-  END_EXCEPTION_HANDLING();
-
- popframe_end:
-  return obj;
-} 
-
-static jobject
-KaffeJNI_NewLocalRef(JNIEnv* env, jobject ref)
-{
-  BEGIN_EXCEPTION_HANDLING(NULL);
-
-  if (ref != NULL)
-    ADD_REF(ref);
-
-  END_EXCEPTION_HANDLING();
-
-  return ref;
+	(*env)->Throw(env, error2Throwable(info));
 }
 
 static jclass
-Kaffe_DefineClass(JNIEnv* env, const char *name, jobject loader, const jbyte* buf, jsize len)
+Kaffe_DefineClass(JNIEnv* env, const char *name UNUSED, jobject loader,
+		  const jbyte* buf, jsize len)
 {
 	Hjava_lang_Class* cls;
 	classFile hand;
 	errorInfo info;
+	jobject loader_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
-	classFileInit(&hand, NULL, buf, (size_t)len, CP_BYTEARRAY);
+	loader_local = unveil(loader); /* save clobbered reg.  */
+
+	classFileInit(&hand, NULL, (unsigned char*)buf, (size_t)len,
+		      CP_BYTEARRAY);
 
 	cls = newClass();
 	if (cls == 0) {
 		postOutOfMemory(&info);
 	} else {
-		cls = readClass(cls, &hand, loader, &info);
+		cls = readClass(cls, &hand, loader_local, &info);
 	}
 	if (cls == 0) {
 		postError(env, &info);
@@ -316,57 +118,147 @@ Kaffe_DefineClass(JNIEnv* env, const char *name, jobject loader, const jbyte* bu
 	return (cls);
 }
 
-/*
- * For this routine, we defer most of the work to Class.forName(),
- * which handles the task of figuring out the right ClassLoader to
- * use based on the calling method, which requires examining the
- * stack backtrace.
- */
-
 static jclass
-Kaffe_FindClass(JNIEnv UNUSED *env, const char* name)
+Kaffe_FindClass(JNIEnv *env UNUSED, const char* name)
 {
-	jstring nameString;
+	stackTraceInfo          *trace;
 	Utf8Const* utf8;
-	jvalue retval;
+	Hjava_lang_ClassLoader  *loader;
+	Hjava_lang_Class	*clazz;
+	errorInfo		einfo;
+	int			i;
+	char*			pathname;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
-	/* We accepts slashes, but Class.forName() does not */
-	utf8 = checkPtr(utf8ConstNew(name, -1));
-	nameString = utf8Const2JavaReplace(utf8, '/', '.');
+	/* convert name to the form used inside the vm */
+	pathname = checkPtr(KMALLOC(strlen (name) + 1));
+	classname2pathname (name, pathname);
+
+	/* create a new utf8 constant */
+	utf8 = utf8ConstNew(pathname, -1);
+
+	/* free the internal form of name */
+	KFREE(pathname);
+
+	/* bail out if we could not create the utf8 constant */
+	if (utf8 == NULL)
+	{
+		postOutOfMemory (&einfo);
+		throwError (&einfo);
+	}
+
+	/* Quote from the JNI documentation:
+	 *
+	 * "In the Java 2 Platform, FindClass locates the class loader associated with the current native method.
+	 *  If the native code belongs to a system class, no class loader will be involved. Otherwise, the proper
+	 *  class loader will be invoked to load and link the named class. When FindClass is called through the
+	 *  Invocation Interface, there is no current native method or its associated class loader. In that case,
+	 *  the result of ClassLoader.getBaseClassLoader is used."
+	 *
+	 * So we ...
+	 */
+
+	/* ... get the stacktrace ... */
+	trace = (stackTraceInfo *)buildStackTrace (NULL);
+	if (trace == NULL)
+	{
+		postOutOfMemory (&einfo);
+		goto error_out;
+	}
+
+	/* ... find the first java method on the stack ... */
+	for (i=0; trace[i].meth != ENDOFSTACK; i++)
+	{
+		if ((trace[i].meth != NULL) &&
+		    (trace[i].meth->class != NULL))
+			break;
+	}
+
+	/* ... determine the loader to be used ... */
+	if (trace[i].meth == ENDOFSTACK) {
+		jvalue retval;
+
+		do_execute_java_class_method (&retval, "java/lang/ClassLoader",
+					      NULL,
+					      "getSystemClassLoader",
+					      "()Ljava/lang/ClassLoader;");
+
+		loader = (Hjava_lang_ClassLoader *)retval.l;
+	} else {
+		loader = trace[i].meth->class->loader;
+	}
+
+	/* ... and finally call loadArray or loadClass.
+	 *
+	 * Why don't we delegate to Class.forName? Our implementation of
+	 * VMClassLoader.loadClass denies access to classes in internal
+	 * packages like kaffe.lang or gnu.classpath (for security reasons).
+	 * However, when FindClass is invoked from the native method of a
+	 * class defined by the bootstrap loader, that is not correct.
+	 * Therefore, we call loadClass / loadArray directly, which corresponds
+	 * to what getClass() does. 
+	 */
+	if (utf8->data[0] == '[')
+	{
+		clazz = loadArray (utf8, loader, &einfo);
+	}
+	else
+	{
+		clazz = loadClass (utf8, loader, &einfo);
+	}
+
+	if (clazz == NULL)
+	{
+		goto error_out;
+	}
+
+	if (processClass (clazz, CSTATE_COMPLETE, &einfo) == false)
+	{
+		goto error_out;
+	}
+
+	ADD_REF(clazz);
+
 	utf8ConstRelease(utf8);
-	checkPtr(nameString);
-
-	do_execute_java_class_method(&retval, "java.lang.Class", NULL,
-		"forName", "(Ljava/lang/String;)Ljava/lang/Class;", nameString);
-	ADD_REF(retval.l);
-
 	END_EXCEPTION_HANDLING();
-	return (retval.l);
+	return (clazz);
+
+error_out:
+	utf8ConstRelease(utf8);
+	throwError (&einfo);
+
+	/* This is to make gcc silent on one warning */
+	return NULL;
 }
 
 static jclass
 Kaffe_GetSuperClass(JNIEnv* env UNUSED, jclass cls)
 {
-	jclass clz;
+	jclass clz, cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
-	clz = ((Hjava_lang_Class*)cls)->superclass;
+	cls_local = unveil(cls); /* save clobbered reg.  */
+	clz = ((Hjava_lang_Class*)cls_local)->superclass;
 
 	END_EXCEPTION_HANDLING();
 	return (clz);
 }
 
-static jbool
+static jboolean
 Kaffe_IsAssignableFrom(JNIEnv* env UNUSED, jclass cls1, jclass cls2)
 {
-	jbool r;
+	jboolean r;
+	jclass cls1_local;
+	jclass cls2_local;
 
 	BEGIN_EXCEPTION_HANDLING(0);
 
-	if (instanceof(cls2, cls1) != 0) {
+	cls1_local = unveil(cls1);
+	cls2_local = unveil(cls2);
+
+	if (instanceof(cls2_local, cls1_local) != 0) {
 		r = JNI_TRUE;
 	}
 	else {
@@ -379,13 +271,17 @@ Kaffe_IsAssignableFrom(JNIEnv* env UNUSED, jclass cls1, jclass cls2)
 static jint
 Kaffe_Throw(JNIEnv* env UNUSED, jobject obj)
 {
+	jobject obj_local;
 	BEGIN_EXCEPTION_HANDLING(0);
 
 	if( obj )
 	{
-		assert(((Hjava_lang_Object *)obj)->vtable);
-		
-		thread_data->exceptObj = (struct Hjava_lang_Throwable*)obj;
+		obj_local = unveil(obj);
+
+		assert(((Hjava_lang_Object *)obj_local)->vtable);
+
+		thread_data->exceptObj =
+		  (struct Hjava_lang_Throwable*)obj_local;
 	}
 
 	END_EXCEPTION_HANDLING();
@@ -396,10 +292,12 @@ static jint
 Kaffe_ThrowNew(JNIEnv* env UNUSED, jclass cls, const char* mess)
 {
 	Hjava_lang_Object* eobj;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(0);
 
-	eobj = execute_java_constructor(NULL, NULL, cls,
+	cls_local = unveil(cls);
+	eobj = execute_java_constructor(NULL, NULL, cls_local,
 					"(Ljava/lang/String;)V",
 					checkPtr(stringC2Java(mess)));
 
@@ -442,11 +340,45 @@ Kaffe_ExceptionCheck(JNIEnv* env UNUSED)
 static void
 Kaffe_ExceptionDescribe(JNIEnv* env UNUSED)
 {
+	const char* cname;
+	Hjava_lang_Class* class;
+	Hjava_lang_Throwable *eobj;
+
 	BEGIN_EXCEPTION_HANDLING_VOID();
 
-	if (thread_data->exceptObj != 0) {
-		do_execute_java_method(NULL, thread_data->exceptObj, "printStackTrace", "()V",
-				       NULL, 0, thread_data->exceptObj); 
+	eobj = thread_data->exceptObj;
+
+	while (eobj != NULL) {
+		/* Don't use the java stack printer because the exception
+		 * may arise in the IO codec.
+		 */
+	       Hjava_lang_String *msg;
+	       char *realname;
+
+	       class = OBJECT_CLASS(&eobj->base);
+	       cname = CLASS_CNAME(class);
+	       realname = KMALLOC(strlen(cname)+1);
+	       pathname2classname(cname, realname);
+			       
+	       msg = unhand(eobj)->detailMessage;
+	       if (msg != NULL) {
+		       char *cmsg = checkPtr(stringJava2C(msg));
+
+		       kprintf(stderr, "%s: %s\n", realname, cmsg);
+		       KFREE(cmsg);
+		       unhand(eobj)->detailMessage = NULL;
+	       } else {
+		       kprintf(stderr, "%s\n", realname);
+	       }
+       	       KFREE(realname);
+	       
+	       printStackTrace (eobj, NULL, true);
+
+	       if (eobj->cause != eobj) {
+		       eobj = eobj->cause;
+		       kprintf(stderr, "caused by: ");
+	       } else
+		       eobj = NULL;
 	}
 	END_EXCEPTION_HANDLING();
 }
@@ -466,10 +398,12 @@ Kaffe_AllocObject(JNIEnv* env UNUSED, jclass cls)
 {
 	jobject obj;
 	Hjava_lang_Class* clazz;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
-	clazz = (Hjava_lang_Class*)cls;
+	cls_local = unveil(cls);
+	clazz = (Hjava_lang_Class*)cls_local;
 
 	if (CLASS_IS_INTERFACE(clazz) || CLASS_IS_ABSTRACT(clazz)) {
 		throwException(InstantiationException(clazz->name->data));
@@ -488,10 +422,12 @@ Kaffe_NewObjectV(JNIEnv* env UNUSED, jclass cls, jmethodID meth, va_list args)
 	Hjava_lang_Class* clazz;
 	jvalue retval;
 	Method* m = (Method*)meth;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
-	clazz = (Hjava_lang_Class*)cls;
+	cls_local = unveil(cls);
+	clazz = (Hjava_lang_Class*)cls_local;
 
 	if (CLASS_IS_INTERFACE(clazz) || CLASS_IS_ABSTRACT(clazz) || !METHOD_IS_CONSTRUCTOR(m)) {
 		throwException(InstantiationException(clazz->name->data));
@@ -510,11 +446,14 @@ Kaffe_NewObject(JNIEnv* env UNUSED, jclass cls, jmethodID meth, ...)
 {
 	jobject obj;
 	va_list args;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
+	cls_local = unveil(cls);
+
 	va_start(args, meth);
-	obj = Kaffe_NewObjectV(env, cls, meth, args);
+	obj = Kaffe_NewObjectV(env, cls_local, meth, args);
 	va_end(args);
 
 	END_EXCEPTION_HANDLING();
@@ -528,10 +467,12 @@ Kaffe_NewObjectA(JNIEnv* env UNUSED, jclass cls, jmethodID meth, jvalue* args)
 	Hjava_lang_Class* clazz;
 	jvalue retval;
 	Method* m = (Method*)meth;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
-	clazz = (Hjava_lang_Class*)cls;
+	cls_local = unveil(cls);
+	clazz = (Hjava_lang_Class*)cls_local;
 
 	if (CLASS_IS_INTERFACE(clazz) || CLASS_IS_ABSTRACT(clazz) || !METHOD_IS_CONSTRUCTOR(m)) {
 		throwException(InstantiationException(clazz->name->data));
@@ -549,23 +490,30 @@ static jclass
 Kaffe_GetObjectClass(JNIEnv* env UNUSED, jobject obj)
 {
 	jclass cls;
+	jobject obj_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
 
-	cls = ((Hjava_lang_Object*)obj)->vtable->class;
+	obj_local = unveil(obj);
+	cls = ((Hjava_lang_Object*)obj_local)->vtable->class;
 
 	END_EXCEPTION_HANDLING();
 	return (cls);
 }
 
-static jbool
+static jboolean
 Kaffe_IsInstanceOf(JNIEnv* env UNUSED, jobject obj, jclass cls)
 {
-	jbool r;
+	jboolean r;
+	jobject obj_local;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(0);
 
-	if (soft_instanceof((Hjava_lang_Class*)cls, (Hjava_lang_Object*)obj) != 0) {
+	obj_local = unveil(obj);
+	cls_local = unveil(cls);
+	if (soft_instanceof((Hjava_lang_Class*)cls_local,
+			    (Hjava_lang_Object*)obj_local) != 0) {
 		r = JNI_TRUE;
 	}
 	else {
@@ -581,19 +529,24 @@ Kaffe_GetMethodID(JNIEnv* env, jclass cls, const char* name, const char* sig)
 {
 	Method* meth;
 	errorInfo info;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
-	meth = lookupClassMethod((Hjava_lang_Class*)cls, name, sig, &info);
+	cls_local = unveil(cls);
+
+	meth = lookupClassMethod((Hjava_lang_Class*)cls_local, name, sig,
+				 &info);
 	if (meth == NULL) {
 		postError(env, &info);
-	} 
+	}
 	else if (METHOD_IS_STATIC(meth)) {
-		postExceptionMessage(&info, JAVA_LANG(NoSuchMethodError), "%s", name);
+		postExceptionMessage(&info, JAVA_LANG(NoSuchMethodError),
+				     "%s", name);
 		postError(env, &info);
 		meth = NULL;
 	}
 	END_EXCEPTION_HANDLING();
-	
+
 	return ((jmethodID)meth);
 }
 
@@ -604,10 +557,14 @@ Kaffe_GetFieldID(JNIEnv* env, jclass cls, const char* name, const char* sig UNUS
 	Field* fld;
 	errorInfo info;
 	Utf8Const* utf8;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
+
+	cls_local = unveil(cls);
 	utf8 = checkPtr(utf8ConstNew(name, -1));
-	fld = lookupClassField((Hjava_lang_Class*)cls, utf8, false, &info);
+	fld = lookupClassField((Hjava_lang_Class*)cls_local, utf8, false,
+			       &info);
 	utf8ConstRelease(utf8);
 	if (fld == NULL) {
 		postError(env, &info);
@@ -621,9 +578,13 @@ Kaffe_GetStaticMethodID(JNIEnv* env, jclass cls, const char* name, const char* s
 {
 	Method* meth;
 	errorInfo info;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
-	meth = lookupClassMethod((Hjava_lang_Class*)cls, name, sig, &info);
+
+	cls_local = unveil(cls);
+	meth = lookupClassMethod((Hjava_lang_Class*)cls_local, name, sig,
+				 &info);
 	if (meth == NULL) {
 		postError(env, &info);
 	} else if (!METHOD_IS_STATIC(meth)) {
@@ -642,10 +603,13 @@ Kaffe_GetStaticFieldID(JNIEnv* env, jclass cls, const char* name, const char* si
 	Field* fld;
 	errorInfo info;
 	Utf8Const* utf8;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(NULL);
+
+	cls_local = unveil(cls);
 	utf8 = checkPtr(utf8ConstNew(name, -1));
-	fld = lookupClassField((Hjava_lang_Class*)cls, utf8, true, &info);
+	fld = lookupClassField((Hjava_lang_Class*)cls_local, utf8, true, &info);
 	utf8ConstRelease(utf8);
 	if (fld == NULL) {
 		postError(env, &info);
@@ -659,9 +623,11 @@ static jsize
 Kaffe_GetArrayLength(JNIEnv* env UNUSED, jarray arr)
 {
 	jsize len;
+	jarray arr_local;
 	BEGIN_EXCEPTION_HANDLING(0);
 
-	len = obj_length((HArrayOfObject*)arr);
+	arr_local = unveil(arr);
+	len = obj_length((HArrayOfObject*)arr_local);
 
 	END_EXCEPTION_HANDLING();
 	return (len);
@@ -672,13 +638,15 @@ Kaffe_RegisterNatives(JNIEnv* env UNUSED, jclass cls, const JNINativeMethod* met
 {
 	Method* meth;
 	int nmeth;
-	int i;
-	int j;
+	int i, j;
+	jclass cls_local;
 
 	BEGIN_EXCEPTION_HANDLING(0);
 
-	meth = CLASS_METHODS((Hjava_lang_Class*)cls);
-	nmeth = CLASS_NMETHODS((Hjava_lang_Class*)cls);
+	cls_local = unveil(cls);
+
+	meth = CLASS_METHODS((Hjava_lang_Class*)cls_local);
+	nmeth = CLASS_NMETHODS((Hjava_lang_Class*)cls_local);
 
 	for (j = 0; j < nmethods; j++) {
 		for (i = 0; i < nmeth; i++) {
@@ -711,10 +679,13 @@ Kaffe_UnregisterNatives(JNIEnv* env UNUSED, jclass cls UNUSED)
 static jint
 Kaffe_MonitorEnter(JNIEnv* env UNUSED, jobject obj)
 {
+	jobject obj_local;
+
 	BEGIN_EXCEPTION_HANDLING(0);
 
 	/* We should never throw out of a JNI call */
-	lockObject(obj);
+	obj_local = unveil(obj);
+	lockObject(obj_local);
 
 	END_EXCEPTION_HANDLING();
 	return (0);
@@ -723,9 +694,12 @@ Kaffe_MonitorEnter(JNIEnv* env UNUSED, jobject obj)
 static jint
 Kaffe_MonitorExit(JNIEnv* env UNUSED, jobject obj)
 {
+	jobject obj_local;
+
 	BEGIN_EXCEPTION_HANDLING(0);
 
-	unlockObject(obj);
+	obj_local = unveil(obj);
+	unlockObject(obj_local);
 
 	END_EXCEPTION_HANDLING();
 	return (0);
@@ -753,7 +727,7 @@ Kaffe_AttachCurrentThread(JavaVM* vm UNUSED, void** penv, void* args UNUSED)
 {
 	if (KTHREAD(attach_current_thread) (false)) {
 		KSEM(init)(&THREAD_DATA()->sem);
-		attachFakedThreadInstance ("test attach", false);
+		KaffeVM_attachFakedThreadInstance ("test attach", false);
 		*penv = THREAD_JNIENV();
 		return 0;
 	}
@@ -765,7 +739,7 @@ Kaffe_AttachCurrentThreadAsDaemon(JavaVM* vm UNUSED, void** penv, void* args UNU
 {
 	if (KTHREAD(attach_current_thread) (true)) {
 		KSEM(init)(&THREAD_DATA()->sem);
-		attachFakedThreadInstance ("daemon attach", true);
+		KaffeVM_attachFakedThreadInstance ("daemon attach", true);
 		*penv = THREAD_JNIENV();
 	        return 0;
 	}
@@ -797,7 +771,7 @@ Kaffe_GetEnv(JavaVM* vm, void** penv, jint interface_id)
 	   given VM. This gets the JavaVM to which the current thread
 	   is attached.  I *think* this is a good way to do this. */
 	Kaffe_GetJavaVM(je, &currentVM);
-	if (!Kaffe_IsSameObject(je, currentVM, vm))
+	if (!KaffeJNI_IsSameObject(je, currentVM, vm))
 		return (JNI_EDETACHED);
 
 	/* Is the requested version of the interface known? */
@@ -808,6 +782,9 @@ Kaffe_GetEnv(JavaVM* vm, void** penv, jint interface_id)
 	case JNI_VERSION_1_2:
 		(*penv) = je;
 		return (JNI_OK);
+	case JNI_VERSION_1_4:
+		(*penv) = je;
+		return (JNI_OK);
 #if 0
 	case JVMDI_VERSION_1:
 		(*penv) = (JVMDI_Interface_1*)&Kaffe_JVMDIEnv;
@@ -817,6 +794,7 @@ Kaffe_GetEnv(JavaVM* vm, void** penv, jint interface_id)
 #if defined(ENABLE_JVMPI)
 	case JVMPI_VERSION_1:
 	case JVMPI_VERSION_1_1:
+	case JVMPI_VERSION_1_2:
 		(*penv) = jvmpiCreateInterface(interface_id);
 		return (JNI_OK);
 #endif
@@ -849,13 +827,13 @@ struct JNINativeInterface Kaffe_JNINativeInterface = {
 	Kaffe_ExceptionOccurred,
 	Kaffe_ExceptionDescribe,
 	Kaffe_ExceptionClear,
-	Kaffe_FatalError,
+	KaffeJNI_FatalError,
 	KaffeJNI_PushLocalFrame,
 	KaffeJNI_PopLocalFrame,
-	Kaffe_NewGlobalRef,
-	Kaffe_DeleteGlobalRef,
-	Kaffe_DeleteLocalRef,
-	Kaffe_IsSameObject,
+	KaffeJNI_NewGlobalRef,
+	KaffeJNI_DeleteGlobalRef,
+	KaffeJNI_DeleteLocalRef,
+	KaffeJNI_IsSameObject,
 	KaffeJNI_NewLocalRef,
 	KaffeJNI_EnsureLocalCapacity,
 	Kaffe_AllocObject,
@@ -1055,14 +1033,14 @@ struct JNINativeInterface Kaffe_JNINativeInterface = {
 	KaffeJNI_GetStringUTFRegion,
 	KaffeJNI_GetPrimitiveArrayCritical,
 	KaffeJNI_ReleasePrimitiveArrayCritical,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
+	KaffeJNI_GetStringChars,
+	KaffeJNI_ReleaseStringChars,
+	KaffeJNI_NewWeakGlobalRef, //
+	KaffeJNI_DeleteWeakGlobalRef, //
 	Kaffe_ExceptionCheck,
-	NULL,
-	NULL,
-	NULL,
+	KaffeJNI_NewDirectByteBuffer,
+	KaffeJNI_GetDirectBufferAddress,
+	KaffeJNI_GetDirectBufferCapacity
 
 };
 
@@ -1111,6 +1089,8 @@ KaffeVM_Arguments Kaffe_JavaVMInitArgs = {
 	ALLOC_HEAPSIZE,	/* Inc heap size */
 	NULL,		/* Class home */
 	NULL,		/* Library home */
+	NULL,           /* No profiler */
+	NULL            /* No arguments to profiler */
 };
 
 /*
