@@ -22,6 +22,12 @@ jmethodID KaffeJNI_FromReflectedMethod (JNIEnv *, jobject);
 jfieldID  KaffeJNI_FromReflectedField (JNIEnv *, jobject);
 jobject   KaffeJNI_ToReflectedMethod (JNIEnv *, jclass, jmethodID, jboolean);
 jobject   KaffeJNI_ToReflectedField (JNIEnv *, jclass, jfieldID, jboolean);
+jobject   KaffeJNI_NewDirectByteBuffer(JNIEnv *, void *, jlong);
+void *    KaffeJNI_GetDirectBufferAddress(JNIEnv *, jobject);
+jlong     KaffeJNI_GetDirectBufferCapacity(JNIEnv *, jobject);
+void NONRETURNING
+          KaffeJNI_FatalError(JNIEnv* env, const char* mess);
+
 
 /* ====== String handling =========================================================== */
 
@@ -32,8 +38,8 @@ const jchar*
         KaffeJNI_GetStringChars(JNIEnv* env, jstring data, jboolean* copy);
 jstring KaffeJNI_NewStringUTF(JNIEnv* env, const char* data);
 jsize   KaffeJNI_GetStringUTFLength(JNIEnv* env, jstring data);
-const jbyte*
-        KaffeJNI_GetStringUTFChars(JNIEnv* env, jstring data, jbool* copy);
+const char*
+        KaffeJNI_GetStringUTFChars(JNIEnv* env, jstring data, jboolean* copy);
 void    KaffeJNI_ReleaseStringUTFChars(JNIEnv* env, jstring data, const char* chars);
 void    KaffeJNI_GetStringRegion(JNIEnv *env, jstring data, jsize start, jsize len, jchar *buf);
 void    KaffeJNI_GetStringUTFRegion(JNIEnv *env, jstring data, jsize start, jsize len, char *buf);
@@ -62,17 +68,17 @@ jfloatArray
 jdoubleArray
         KaffeJNI_NewDoubleArray(JNIEnv* env, jsize len);
 
-jboolean* KaffeJNI_GetBooleanArrayElements(JNIEnv* env, jbooleanArray arr, jbool* iscopy);
-jbyte*    KaffeJNI_GetByteArrayElements(JNIEnv* env, jbyteArray arr, jbool* iscopy);
-void*     KaffeJNI_GetPrimitiveArrayCritical(JNIEnv* env, jarray arr, jbool* iscopy);
-jchar*    KaffeJNI_GetCharArrayElements(JNIEnv* env, jcharArray arr, jbool* iscopy);
-jshort*   KaffeJNI_GetShortArrayElements(JNIEnv* env, jshortArray arr, jbool* iscopy);
-jint*     KaffeJNI_GetIntArrayElements(JNIEnv* env, jintArray arr, jbool* iscopy);
-jlong*    KaffeJNI_GetLongArrayElements(JNIEnv* env, jlongArray arr, jbool* iscopy);
-jfloat*   KaffeJNI_GetFloatArrayElements(JNIEnv* env, jfloatArray arr, jbool* iscopy);
-jdouble*  KaffeJNI_GetDoubleArrayElements(JNIEnv* env, jdoubleArray arr, jbool* iscopy);
+jboolean* KaffeJNI_GetBooleanArrayElements(JNIEnv* env, jbooleanArray arr, jboolean* iscopy);
+jbyte*    KaffeJNI_GetByteArrayElements(JNIEnv* env, jbyteArray arr, jboolean* iscopy);
+void*     KaffeJNI_GetPrimitiveArrayCritical(JNIEnv* env, jarray arr, jboolean* iscopy);
+jchar*    KaffeJNI_GetCharArrayElements(JNIEnv* env, jcharArray arr, jboolean* iscopy);
+jshort*   KaffeJNI_GetShortArrayElements(JNIEnv* env, jshortArray arr, jboolean* iscopy);
+jint*     KaffeJNI_GetIntArrayElements(JNIEnv* env, jintArray arr, jboolean* iscopy);
+jlong*    KaffeJNI_GetLongArrayElements(JNIEnv* env, jlongArray arr, jboolean* iscopy);
+jfloat*   KaffeJNI_GetFloatArrayElements(JNIEnv* env, jfloatArray arr, jboolean* iscopy);
+jdouble*  KaffeJNI_GetDoubleArrayElements(JNIEnv* env, jdoubleArray arr, jboolean* iscopy);
 
-void      KaffeJNI_ReleaseBooleanArrayElements(JNIEnv* env, jbooleanArray arr, jbool* elems, jint mode);
+void      KaffeJNI_ReleaseBooleanArrayElements(JNIEnv* env, jbooleanArray arr, jboolean* elems, jint mode);
 void      KaffeJNI_ReleaseByteArrayElements(JNIEnv* env, jbyteArray arr, jbyte* elems, jint mode);
 void      KaffeJNI_ReleasePrimitiveArrayCritical(JNIEnv* env, jbyteArray arr, void* elems, jint mode);
 void      KaffeJNI_ReleaseCharArrayElements(JNIEnv* env, jcharArray arr, jchar* elems, jint mode);
@@ -82,7 +88,7 @@ void      KaffeJNI_ReleaseLongArrayElements(JNIEnv* env, jlongArray arr, jlong*
 void      KaffeJNI_ReleaseFloatArrayElements(JNIEnv* env, jfloatArray arr, jfloat* elems, jint mode);
 void      KaffeJNI_ReleaseDoubleArrayElements(JNIEnv* env, jdoubleArray arr, jdouble* elems, jint mode);
 
-void      KaffeJNI_GetBooleanArrayRegion(JNIEnv* env, jbooleanArray arr, jsize start, jsize len, jbool* data);
+void      KaffeJNI_GetBooleanArrayRegion(JNIEnv* env, jbooleanArray arr, jsize start, jsize len, jboolean* data);
 void      KaffeJNI_GetByteArrayRegion(JNIEnv* env, jbyteArray arr, jsize start, jsize len, jbyte* data);
 void      KaffeJNI_GetCharArrayRegion(JNIEnv* env, jcharArray arr, jsize start, jsize len, jchar* data);
 void      KaffeJNI_GetShortArrayRegion(JNIEnv* env, jshortArray arr, jsize start, jsize len, jshort* data);
@@ -91,7 +97,7 @@ void      KaffeJNI_GetLongArrayRegion(JNIEnv* env, jlongArray arr, jsize start,
 void      KaffeJNI_GetFloatArrayRegion(JNIEnv* env, jfloatArray arr, jsize start, jsize len, jfloat* data);
 void      KaffeJNI_GetDoubleArrayRegion(JNIEnv* env, jdoubleArray arr, jsize start, jsize len, jdouble* data);
 
-void      KaffeJNI_SetBooleanArrayRegion(JNIEnv* env, jbooleanArray arr, jsize start, jsize len, jbool* data);
+void      KaffeJNI_SetBooleanArrayRegion(JNIEnv* env, jbooleanArray arr, jsize start, jsize len, jboolean* data);
 void      KaffeJNI_SetByteArrayRegion(JNIEnv* env, jbyteArray arr, jsize start, jsize len, jbyte* data);
 void      KaffeJNI_SetCharArrayRegion(JNIEnv* env, jcharArray arr, jsize start, jsize len, jchar* data);
 void      KaffeJNI_SetShortArrayRegion(JNIEnv* env, jshortArray arr, jsize start, jsize len, jshort* data);
@@ -115,7 +121,7 @@ jdouble   KaffeJNI_GetDoubleField(JNIEnv* env, jobject obj, jfieldID fld);
 /* ====== Functions to set fields =================================================== */
 
 void      KaffeJNI_SetObjectField(JNIEnv* env, jobject obj, jfieldID fld, jobject val);
-void      KaffeJNI_SetBooleanField(JNIEnv* env, jobject obj, jfieldID fld, jbool val);
+void      KaffeJNI_SetBooleanField(JNIEnv* env, jobject obj, jfieldID fld, jboolean val);
 void      KaffeJNI_SetByteField(JNIEnv* env, jobject obj, jfieldID fld, jbyte val);
 void      KaffeJNI_SetCharField(JNIEnv* env, jobject obj, jfieldID fld, jchar val);
 void      KaffeJNI_SetShortField(JNIEnv* env, jobject obj, jfieldID fld, jshort val);
@@ -139,7 +145,7 @@ jdouble   KaffeJNI_GetStaticDoubleField(JNIEnv* env, jclass cls, jfieldID fld);
 /* ====== Functions to set static fields ============================================ */
 
 void     KaffeJNI_SetStaticObjectField(JNIEnv* env, jclass cls, jfieldID fld, jobject val);
-void     KaffeJNI_SetStaticBooleanField(JNIEnv* env, jclass cls, jfieldID fld, jbool val);
+void     KaffeJNI_SetStaticBooleanField(JNIEnv* env, jclass cls, jfieldID fld, jboolean val);
 void     KaffeJNI_SetStaticByteField(JNIEnv* env, jclass cls, jfieldID fld, jbyte val);
 void     KaffeJNI_SetStaticCharField(JNIEnv* env, jclass cls, jfieldID fld, jchar val);
 void     KaffeJNI_SetStaticShortField(JNIEnv* env, jclass cls, jfieldID fld, jshort val);
@@ -270,4 +276,17 @@ jobject  KaffeJNI_CallStaticObjectMethodV(JNIEnv* env, jclass cls, jmethodID met
 jobject  KaffeJNI_CallStaticObjectMethod(JNIEnv* env, jclass cls, jmethodID meth, ...);
 jobject  KaffeJNI_CallStaticObjectMethodA(JNIEnv* env, jclass cls, jmethodID meth, jvalue* args);
 
+/* ====== Functions to handle java references ======================================= */
+
+void     KaffeJNI_DeleteGlobalRef(JNIEnv* env, jref obj);
+void     KaffeJNI_DeleteLocalRef(JNIEnv* env, jref obj);
+jboolean KaffeJNI_IsSameObject(JNIEnv* env, jobject obj1, jobject obj2);
+jref     KaffeJNI_NewGlobalRef(JNIEnv* env, jref obj);
+jint     KaffeJNI_EnsureLocalCapacity(JNIEnv* env, jint capacity);
+jobject  KaffeJNI_PopLocalFrame(JNIEnv* env, jobject obj);
+jobject  KaffeJNI_NewLocalRef(JNIEnv* env, jobject ref);
+jweak    KaffeJNI_NewWeakGlobalRef(JNIEnv *env, jobject obj);
+void     KaffeJNI_DeleteWeakGlobalRef(JNIEnv *env, jweak obj);
+jint     KaffeJNI_PushLocalFrame(JNIEnv* env, jint capacity);
+
 #endif
