@@ -2,9 +2,12 @@
  * The garbage collector.
  * The name is misleading.  GC is non-incremental at this point.
  *
- * Copyright (c) 1996, 1997, 2004
+ * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
+ * Copyright (c) 2003, 2004
+ *	Kaffe.org contributors. See ChangeLog for details. All rights reserved.
+ *   
  * See the file "license.terms" for information on usage and redistribution 
  * of this file. 
  */
@@ -87,7 +90,6 @@ static counter gcfixedmem;
         ((uintp) (from) >= gc_get_heap_base() && \
 	 (uintp) (from) < gc_get_heap_base() + gc_get_heap_range())
 
-static void *reserve;
 static void *outOfMem;
 static void *outOfMem_allocator;
 
@@ -338,7 +340,9 @@ DBG(GCWALK,
 	record_marked(1, size);
 
 	if (size > 0) {
-		for (mem = ((const int8*)base) + (size & (uintp)-ALIGNMENTOF_VOIDP) - sizeof(void*);
+		uintp alignment = ALIGNMENTOF_VOIDP_IN_STACK;
+
+		for (mem = ((const int8*)base) + (size & -alignment) - sizeof(void*);
 		     (const void*)mem >= base;
 		     mem -= ALIGNMENTOF_VOIDP) {
 			const void *p = *(void * const *)mem;
@@ -635,7 +639,7 @@ DBG(GCSTAT,
 			(gcStats.totalmem-gcStats.freedmem)/1024, 
 			gcStats.totalobj,
 			gcStats.totalobj-gcStats.freedobj,
-			(1.0 - ((gcStats.totalmem-gcStats.freedmem)/
+			(1.0 - (((float)gcStats.totalmem-gcStats.freedmem)/
 				gc_get_heap_total())) * 100.0,
 			gcStats.allocmem/1024,
 			gcStats.allocobj,
@@ -680,11 +684,11 @@ startGC(Collector *gcif)
 	gcStats.markedmem = 0;
 
 #if defined(ENABLE_JVMPI)
-	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_KGC_START) )
+	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_GC_START) )
 	{
 		JVMPI_Event ev;
 
-		ev.event_type = JVMPI_EVENT_KGC_START;
+		ev.event_type = JVMPI_EVENT_GC_START;
 		jvmpiPostEvent(&ev);
 	}
 #endif
@@ -786,6 +790,7 @@ finishGC(Collector *gcif)
 
 	while (toRemove.cnext != &toRemove) {
 		destroy_func_t destroy;
+
 		unit = toRemove.cnext; 
 		info = gc_mem2block(unit);
 		idx = GCMEM2IDX(info, unit);
@@ -805,7 +810,9 @@ finishGC(Collector *gcif)
 		}
 #endif
 
-		/* clear all weak references to the object */
+		/* clear all weak references to the object if it has not already been
+		 * during the finalisation mark phase.
+		 */
 		KaffeGC_clearWeakRef(gcif, UTOMEM(unit));
 
 		/* invoke destroy function before freeing the object */
@@ -823,11 +830,11 @@ finishGC(Collector *gcif)
 	}
 
 #if defined(ENABLE_JVMPI)
-	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_KGC_FINISH) )
+	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_GC_FINISH) )
 	{ 
 		JVMPI_Event ev;
 
-		ev.event_type = JVMPI_EVENT_KGC_FINISH;
+		ev.event_type = JVMPI_EVENT_GC_FINISH;
 		ev.u.gc_info.used_objects = (jlong)gcStats.markedobj;
 		ev.u.gc_info.used_object_space = (jlong)gcStats.markedmem;
 		ev.u.gc_info.total_object_space = (jlong)gcStats.totalmem;
@@ -873,8 +880,7 @@ static void finaliserJob(Collector *gcif)
   gc_block* info = NULL;
   gc_unit* unit = NULL;
   int idx = 0;
-  void *stack;
-#define iLockRoot (*where)
+  int func = 0;
 
   /*
    * Loop until the list of objects whose finaliser needs to be run is empty
@@ -901,16 +907,29 @@ static void finaliserJob(Collector *gcif)
    */
   while (gclists[finalise].cnext != &gclists[finalise]) {
     unit = gclists[finalise].cnext;
+    lockStaticMutex(&gc_lock);
     info = gc_mem2block(unit);
     idx = GCMEM2IDX(info, unit);
-    
+    func = KGC_GET_FUNCS(info, idx); 
+    unlockStaticMutex(&gc_lock);
+
+    /* Clear weak references to this object. Because according to the Java API spec.
+     * "Suppose that the garbage collector determines at a certain point in time 
+     * that an object is weakly reachable. At that time it will atomically clear
+     * all weak references to that object and all weak references to any other
+     * weakly-reachable objects from which that object is reachable through a chain
+     * of strong and soft references."
+     */
+    KaffeGC_clearWeakRef(gcif, UTOMEM(unit));
+
     /* Call finaliser */
     unlockStaticMutex(&finman);
-    (*gcFunctions[KGC_GET_FUNCS(info,idx)].final)(gcif, UTOMEM(unit));
+    (*gcFunctions[func].final)(gcif, UTOMEM(unit));
     lockStaticMutex(&finman);
     
     /* and remove unit from the finaliser list */
     lockStaticMutex(&gc_lock);
+    info = gc_mem2block(unit);
     UREMOVELIST(unit);
     UAPPENDLIST(gclists[nofin_white], unit);
     
@@ -926,14 +945,6 @@ static void finaliserJob(Collector *gcif)
   info = NULL;
   unit = NULL;
   idx = 0;
-
-  /* This cleans STACK_SWEEP_MARGIN bytes on the stack. This is needed
-   * because some objects may still lie on our stack.
-   */
-  stack = alloca(STACK_SWEEP_MARGIN);
-
-  memset((void *)((uintp)stack), 0, STACK_SWEEP_MARGIN);
-#undef iLockRoot
 }
 
 static void NONRETURNING
@@ -1044,7 +1055,7 @@ void throwOutOfMemory(void) NONRETURNING;
 
 static
 void*
-gcMalloc(Collector* gcif UNUSED, size_t size, gc_alloc_type_t fidx)
+gcMalloc(Collector* gcif, size_t size, gc_alloc_type_t fidx)
 {
 	gc_block* info;
 	gc_unit* unit;
@@ -1056,6 +1067,7 @@ gcMalloc(Collector* gcif UNUSED, size_t size, gc_alloc_type_t fidx)
 	assert(gc_init != 0);
 	assert(gcFunctions[fidx].description != NULL);
 	assert(size != 0);
+	assert(size > 0);
 
 	size += sizeof(gc_unit);
 
@@ -1073,7 +1085,7 @@ gcMalloc(Collector* gcif UNUSED, size_t size, gc_alloc_type_t fidx)
 				/* Try invoking GC if it is available */
 				if (garbageman != 0) {
 					unlockStaticMutex(&gc_lock);
-					adviseGC();
+					KGC_invoke(gcif, 0);
 					lockStaticMutex(&gc_lock);
 				}
 				break;
@@ -1082,7 +1094,7 @@ gcMalloc(Collector* gcif UNUSED, size_t size, gc_alloc_type_t fidx)
 				/* Grow the heap */
 				DBG (GCSYSALLOC, dprintf ("growing heap by %u bytes of type %s (%2.1f%% free)\n", 
 							  (unsigned int)size, gcFunctions[fidx].description,
-							  (1.0 - (gcStats.totalmem / gc_get_heap_total())) * 100.0); );
+							  (1.0 - ((double)gcStats.totalmem / gc_get_heap_total())) * 100.0); );
 				
 				gc_heap_grow(size);
 				break;
@@ -1197,9 +1209,7 @@ gcThrowOOM(Collector *gcif UNUSED)
 	 * outOfMemory error.  We can use some or all of the reserved
 	 * pages to actually grab an error.
 	 */
-	if (reserve) {
-		gc_primitive_free(reserve);
-		reserve = NULL;
+	if (gc_primitive_use_reserve()) {
 		if (!ret || ret == OOM_ALLOCATING) {
 			unlockStaticMutex(&gc_lock);
 			ret = OutOfMemoryError; /* implicit allocation */
@@ -1211,7 +1221,7 @@ gcThrowOOM(Collector *gcif UNUSED)
 		unlockStaticMutex(&gc_lock);
 		dprintf(
 			"Not enough memory to throw OutOfMemoryError!\n");
-		ABORT();
+		KAFFEVM_ABORT();
 	}
 	unlockStaticMutex(&gc_lock);
 	if (reffed) gc_rm_ref(ret);
@@ -1427,7 +1437,7 @@ objectSizesPrint(void)
 
         for (i = 0; i < nr; ) {
                 for (j = 0; j < 3 && i < nr; j++, i++) {
-                        dprintf("%10d :%5.1f /%5.1f  ", objectSizes[i].size, 
+                        dprintf("%10zd :%5.1f /%5.1f  ", objectSizes[i].size, 
 				(float)(objectSizes[i].count * 100 / (float)count), 
 				(float)(objectSizes[i].total * 100 / (float)total));
                 }
@@ -1519,7 +1529,7 @@ createGC(void)
   gc_obj.collector.ops = &KGC_Ops;
   
   gc_heap_initialise ();
-  reserve = gc_primitive_reserve ();
+  gc_primitive_reserve(KGC_NUMBER_PAGE_IN_RESERVE);
   
   return (&gc_obj.collector);
 }
