@@ -40,6 +40,8 @@
 #endif
 
 static gc_block *gc_last_block;
+static gc_block *gc_first_block;
+static gc_block *gc_reserve_pages;
 static iStaticLock	gc_heap_lock;
 
 #if defined(KAFFE_STATS)
@@ -126,6 +128,8 @@ int gc_system_alloc_cnt;
 #define	ROUNDUPPAGESIZE(V)	(((uintp)(V) + gc_pgsize - 1) & -gc_pgsize)
 
 static char * gc_block_base = NULL;
+static size_t gc_num_blocks = 0;
+static size_t gc_num_live_pages = 0;
 
 #define KGC_BLOCKS		((gc_block *) gc_block_base)
 
@@ -230,6 +234,13 @@ gc_heap_check(void)
 
 #endif /* !(defined(NDEBUG) || !defined(KAFFE_VMDEBUG)) */
 
+
+static inline bool
+gc_heap_is_unlimited(void)
+{
+  return gc_heap_limit == UNLIMITED_HEAP;
+}
+
 /*
  * Initialise allocator.
  */
@@ -254,11 +265,12 @@ gc_heap_initialise(void)
 	/*
 	 * Perform some sanity checks.
 	 */
-	if (gc_heap_initial_size > gc_heap_limit) {
+	if ((gc_heap_initial_size > gc_heap_limit) 
+	    && !gc_heap_is_unlimited()) {
 		dprintf(
 		    "Initial heap size (%dK) > Maximum heap size (%dK)\n",
 		    (int) (gc_heap_initial_size/1024), (int)(gc_heap_limit/1024));
-		EXIT(-1);
+		KAFFEVM_EXIT(-1);
 	}
 
 #ifndef PREDEFINED_NUMBER_OF_TILES
@@ -360,8 +372,12 @@ gc_heap_malloc(size_t sz)
 	gc_block** mptr;
 	gc_block* blk;
 	size_t nsz;
+#if defined(KAFFE_STATS)
+	static timespent heap_alloc_time;
+#endif
 
 	lockStaticMutex(&gc_heap_lock);
+	startTiming(&heap_alloc_time, "gc_heap_malloc");
 
 DBG(SLACKANAL,
 	if (KGC_SMALL_OBJECT(sz)) {
@@ -442,6 +458,7 @@ DBG(GCALLOC,	dprintf("gc_heap_malloc: large block %ld at %p\n",
 	assert(KGC_OBJECT_SIZE(mem) >= sz);
 
 	out:
+	stopTiming(&heap_alloc_time);
 	unlockStaticMutex(&gc_heap_lock);
 
 	return (mem);
@@ -458,6 +475,9 @@ gc_heap_free(void* mem)
 	int lnr;
 	int msz;
 	int idx;
+#if defined(KAFFE_STATS)
+	static timespent heap_free_time;
+#endif
 
 	info = gc_mem2block(mem);
 	idx = GCMEM2IDX(info, mem);
@@ -473,6 +493,7 @@ DBG(GCFREE,
 	dprintf("gc_heap_free: memory %p size %d\n", mem, info->size);	);
 
 	lockStaticMutex(&gc_heap_lock);
+	startTiming(&heap_free_time, "gc_heap_free");
 
 	if (KGC_SMALL_OBJECT(info->size)) {
 		lnr = sztable[info->size].list;
@@ -529,6 +550,7 @@ DBG(GCFREE,
 		gc_primitive_free(info);
 	}
 
+	stopTiming(&heap_free_time);
 	unlockStaticMutex(&gc_heap_lock);
 
 DBG(GCDIAG,
@@ -613,7 +635,7 @@ gc_large_block(size_t sz)
 	/* number of pages allocated for block */
 	block_count = msz >> gc_pgbits;
 	
-	DBG(GCPRIM, dprintf ("large block covers %x pages\n", block_count); );
+	DBG(GCPRIM, dprintf ("large block covers %zx pages\n", block_count); );
 
 	/* Setup the meta-data for the block */
 	DBG(GCDIAG, gc_set_magic_marker(info));
@@ -945,14 +967,17 @@ gc_primitive_free(gc_block* mem)
 /*
  * Try to reserve some memory for OOM exception handling.  Gc once at
  * the beginning.  We start out looking for an arbitrary number of
- * pages (4), and cut our expectations in half until we are able to
+ * pages, and cut our expectations in half until we are able to
  * meet them.
  */
-gc_block *
-gc_primitive_reserve(void)
+void
+gc_primitive_reserve(size_t numpages)
 {
 	gc_block *r = NULL;
-	size_t size = 4 * gc_pgsize;
+	size_t size = numpages * gc_pgsize;
+
+	if (gc_reserve_pages != NULL)
+	        return;
 	
 	while (size >= gc_pgsize && !(r = gc_primitive_alloc(size))) {
 		if (size == gc_pgsize) {
@@ -960,9 +985,28 @@ gc_primitive_reserve(void)
 		}
 		size /= 2;
 	}
-	return r;
+	/* As it is done just at the initialization it is expected to have at 
+	 * least one page free */
+	assert(r != NULL);
+	gc_reserve_pages = r;
 }
 
+/*
+ * We return the reserve to the heap if it has not been already used.
+ * This function returns true if some reserve was still available.
+ */
+bool
+gc_primitive_use_reserve()
+{
+        if (gc_reserve_pages == NULL)
+	        return false;
+	gc_primitive_free(gc_reserve_pages);
+	gc_reserve_pages = NULL;
+
+	return true;
+}
+
+
 /*
  * System memory management:  Obtaining additional memory from the
  * OS.  This looks more complicated than it is, since it does not require
@@ -1062,17 +1106,15 @@ static void *
 gc_block_alloc(size_t size)
 {
 	int size_pg = (size>>gc_pgbits);
-	static int n_live = 0;	/* number of pages in java heap */
-	static int nblocks;	/* number of gc_blocks in array */
 	uintp heap_addr;
 	static uintp last_addr;
 
 	if (!gc_block_base) {
-		nblocks = (gc_heap_limit+gc_pgsize-1)>>gc_pgbits;
+		gc_num_blocks = (size+gc_pgsize-1)>>gc_pgbits;
 
-		gc_block_base = malloc(nblocks * sizeof(gc_block));
+		gc_block_base = malloc(gc_num_blocks * sizeof(gc_block));
 		if (!gc_block_base) return NULL;
-		memset(gc_block_base, 0, nblocks * sizeof(gc_block));
+		memset(gc_block_base, 0, gc_num_blocks * sizeof(gc_block));
 	}
 
 	DBG(GCSYSALLOC, dprintf("pagealloc(%ld)", (long) size));
@@ -1086,11 +1128,11 @@ gc_block_alloc(size_t size)
 	}
 
 	if (gc_mem2block((void *) (heap_addr + size))
-	    > ((gc_block *)gc_block_base) + nblocks
+	    > ((gc_block *)gc_block_base) + gc_num_blocks
 	    || heap_addr < gc_heap_base) {
 		char * old_blocks = gc_block_base;
-		int onb = nblocks;
-		int min_nb;	/* minimum size of array to hold heap_addr */
+		int onb = gc_num_blocks;
+		unsigned int min_nb;	/* minimum size of array to hold heap_addr */
 #if defined(KAFFE_STATS)
 		static timespent growtime;
 #endif
@@ -1104,31 +1146,58 @@ gc_block_alloc(size_t size)
 		   currently fit in the gc_block array.  But, we must
 		   also make sure to allocate enough blocks to cover
 		   the current allocation */
-		nblocks = (nblocks * (gc_heap_limit >> gc_pgbits))
-			/ n_live;
+		gc_num_blocks = (gc_num_blocks * ((gc_heap_total + size) >> gc_pgbits))
+			/ gc_num_live_pages;
 		if (heap_addr < gc_heap_base) 
-			min_nb = nblocks
+			min_nb = gc_num_blocks
 			  + ((gc_heap_base - heap_addr) >> gc_pgbits);
 		else
 			min_nb = ((heap_addr + size) - gc_heap_base) >>
 			  gc_pgbits;
-		nblocks = MAX(nblocks, min_nb);
+		gc_num_blocks = MAX(gc_num_blocks, min_nb);
 		DBG(GCSYSALLOC,
 		    dprintf("growing block array from %d to %d elements\n",
-			    onb, nblocks));
+			    onb, gc_num_blocks));
 
 		KTHREAD(spinon)(NULL);
 		gc_block_base = realloc(old_blocks,
-						nblocks * sizeof(gc_block));
+					gc_num_blocks * sizeof(gc_block));
 		if (!gc_block_base) {
-			/* roll back this call */
-			pagefree(heap_addr, size);
-			gc_block_base = old_blocks;
-			nblocks = onb;
-			KTHREAD(spinoff)(NULL);
-			return NULL;
+			/* In some implementations, realloc is not smart enough to acquire new block
+			 * in a non-contiguous region. Even if internally it calls some malloc procedure
+			 * it fails evenly. A countermeasure is to use slow real malloc if realloc fails
+			 * and only if that call also fails we put throw a OOM.
+			 */
+			DBG(GCSYSALLOC, dprintf("realloc has failed. Trying malloc.\n"));
+			gc_block_base = malloc(gc_num_blocks * sizeof(gc_block));
+			if (!gc_block_base) {
+				/* roll back this call */
+				DBG(GCSYSALLOC, dprintf("failed to grow the block list\n"));
+				pagefree(heap_addr, size);
+				gc_block_base = old_blocks;
+				gc_num_blocks = onb;
+				KTHREAD(spinoff)(NULL);
+				return NULL;
+			}
+			memcpy(gc_block_base, old_blocks, onb * sizeof(gc_block));
+			free(old_blocks);
 		}
+		DBG(GCSYSALLOC, dprintf("old block_base = %p, new block_base = %p\n", old_blocks, gc_block_base));
+		if (heap_addr < gc_heap_base) {
+		  int32 i, j, oldBase;
+		  gc_block *b = (gc_block *) gc_block_base;
 
+		  oldBase = (gc_heap_base - heap_addr) >> gc_pgbits;
+
+		  for (i=(onb-1),j=(oldBase+onb-1); i >= 0; i--,j--)
+		  	memcpy(&b[j], &b[i], sizeof(gc_block));
+
+		  memset((gc_block *)gc_block_base, 0,
+		         (gc_num_blocks - onb) * sizeof(gc_block));
+		} else {
+		  memset(((gc_block *)gc_block_base) + onb, 0,
+		         (gc_num_blocks - onb) * sizeof(gc_block));
+		}
 		/* If the array's address has changed, we have to fix
 		   up the pointers in the gc_blocks, as well as all
 		   external pointers to the gc_blocks.  We can only
@@ -1152,22 +1221,26 @@ gc_block_alloc(size_t size)
 				R(gc_freeobj, b[i].free);
 			  }
 
-			memset(b + onb, 0,
-			       (nblocks - onb) * sizeof(gc_block));
-
 			for (i = 0; i<=KGC_PRIM_LIST_COUNT; i++)
 				R(gc_block, gc_prim_freelist[i]);
 
 			for (i = 0; freelist[i].list != (void*)-1; i++) 
 				R(gc_block, freelist[i].list);
+
+			R(gc_block, gc_reserve_pages);
+			R(gc_block, gc_last_block);
+			R(gc_block, gc_first_block);
 #undef R
 		}
 		KTHREAD(spinoff)(NULL);
 		stopTiming(&growtime);
 	}
-	n_live += size_pg;
+	gc_num_live_pages += size_pg;
 	last_addr = MAX(last_addr, heap_addr + size);
 	gc_heap_range = last_addr - gc_heap_base;
+	if (gc_heap_base > heap_addr)
+	  gc_heap_base = heap_addr;
+
 	DBG(GCSYSALLOC, dprintf("%ld unused bytes in heap addr range\n",
 				(long) (gc_heap_range - gc_heap_total)));
 #if defined(KAFFE_VMDEBUG)
@@ -1205,7 +1278,7 @@ gc_heap_grow(size_t sz)
 	if (gc_heap_total == gc_heap_limit) {
 		unlockStaticMutex(&gc_heap_lock);
 		return (NULL);
-	} else if (gc_heap_total + sz > gc_heap_limit) {
+	} else if (gc_heap_total + sz > gc_heap_limit && !gc_heap_is_unlimited()) {
 		/* take as much memory as we can */
 		sz = gc_heap_limit - gc_heap_total;
 		assert(sz % gc_pgsize == 0);
@@ -1226,7 +1299,7 @@ gc_heap_grow(size_t sz)
 	}
 
 	gc_heap_total += sz;
-	assert(gc_heap_total <= gc_heap_limit);
+	assert(gc_heap_total <= gc_heap_limit || gc_heap_is_unlimited());
 
 	/* Place block into the freelist for subsequent use */
 	DBG(GCDIAG, gc_set_magic_marker(blk));
@@ -1234,8 +1307,15 @@ gc_heap_grow(size_t sz)
 
 	/* maintain list of primitive blocks */
 	if (gc_last_block) {
-		gc_last_block->pnext = blk;
-		blk->pprev = gc_last_block;
+		if (gc_last_block < blk) {
+			gc_last_block->pnext = blk;
+			blk->pprev = gc_last_block;
+		} else {
+			assert(gc_first_block->pprev == NULL);
+			gc_first_block->pprev = blk;
+			blk->pnext = gc_first_block;
+			gc_first_block = blk;
+		}
 	}
 	
 	gc_last_block = blk;
