@@ -2,9 +2,12 @@
  * soft.c
  * Soft instruction support.
  *
- * Copyright (c) 1996, 1997, 2004
+ * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
+ * Copyright (c) 2004
+ *	Kaffe.org contributors. See ChangeLog for details. All rights reserved.
+ *   
  * See the file "license.terms" for information on usage and redistribution
  * of this file.
  */
@@ -298,7 +301,7 @@ inline
 jint
 instanceof_interface(Hjava_lang_Class* c, Hjava_lang_Class* oc)
 {
-	int i;
+	unsigned int i;
 	Hjava_lang_Class **impl_clazz;
 
 	if (oc->state < CSTATE_PREPARED || c->state < CSTATE_PREPARED || CLASS_IS_ARRAY(oc) || CLASS_IS_INTERFACE(oc))
@@ -307,7 +310,7 @@ instanceof_interface(Hjava_lang_Class* c, Hjava_lang_Class* oc)
 	     * prepared the dumb way is the only way. Arrays and interface do not have
 	     * any implementors too so we have to go through the all list.
 	     */
-	    for (i = oc->total_interface_len - 1; i >= 0; i--) {
+	    for (i = 0; i < oc->total_interface_len; i++) {
 	      if (c == oc->interfaces[i]) {
 		return 1;
 	      }
@@ -860,12 +863,16 @@ soft_mul(jint v1, jint v2)
 jint
 soft_div(jint v1, jint v2)
 {
+	if (v2 == -1)
+		return -v1;
 	return (v1/v2);
 }
 
 jint
 soft_rem(jint v1, jint v2)
 {
+	if (v2 == -1)
+		return 0;
 	return (v1%v2);
 }
 
@@ -1059,10 +1066,10 @@ soft_trace(Method* meth, void* args UNUSED)
     dprintf("soft_trace: %s.%s%s\n", CLASS_CNAME(meth->class), meth->name->data, METHOD_SIGD(meth));
 }
 
-void
-soft_enter_method(Hjava_lang_Object *obj UNUSED, Method *meth UNUSED)
-{
 #if defined(ENABLE_JVMPI)
+void
+soft_enter_method(Hjava_lang_Object *obj, Method *meth)
+{
 	if( JVMPI_EVENT_ISENABLED(JVMPI_EVENT_METHOD_ENTRY) )
 	{
 		JVMPI_Event ev;
@@ -1080,8 +1087,13 @@ soft_enter_method(Hjava_lang_Object *obj UNUSED, Method *meth UNUSED)
 		ev.u.method_entry2.obj_id = obj;
 		jvmpiPostEvent(&ev);
 	}
-#endif
 }
+#else
+void
+soft_enter_method(Hjava_lang_Object *obj UNUSED, Method *meth UNUSED)
+{
+}
+#endif
 
 #if defined(ENABLE_JVMPI)
 void
