@@ -10,7 +10,7 @@
 #include "config-std.h"
 #include "config-mem.h"
 #include "classMethod.h"
-#include "kaffe/jni_md.h"
+#include "jni_md.h"
 #include "gtypes.h"
 #include "constants.h"
 #include "object.h"
@@ -26,7 +26,7 @@ static hashtab_t	hashTable;	/* intern hash table */
 static iStaticLock	stringLock; /* mutex on all intern operations */
 
 /* Internal functions */
-static int		stringHashValue(void *ptr);
+static int		stringHashValue(const void *ptr);
 static int		stringCompare(const void *s1, const void *s2);
 
 /*
@@ -358,7 +358,7 @@ stringUninternString(Hjava_lang_String* string)
  * is getting written no matter who writes it (strings are immutable).
  */
 static int
-stringHashValue(void *ptr)
+stringHashValue(const void *ptr)
 {
 	Hjava_lang_String *string = (Hjava_lang_String*) ptr;
 	jint hash;
@@ -490,7 +490,7 @@ stringCharArray2Java(const jchar *data, int len)
 void
 stringWalk(Collector* collector, void *gc_info, void* str, uint32 size UNUSED)
 {
-	Hjava_lang_String *objstr = (Hjava_lang_Object *)str;
+	Hjava_lang_String *objstr = (Hjava_lang_String *)str;
 	iLock *lk;
 
         /* That's all we have to do here */
