@@ -12,6 +12,13 @@
 #ifndef __support_h
 #define	__support_h
 
+
+#include "config.h"
+
+#if defined(HAVE_STRING_H)
+#include <string.h>
+#endif
+
 #include "config-std.h"
 #include "gtypes.h"
 #include <stdarg.h>
@@ -40,7 +47,7 @@ extern nativeFunction* native_funcs;
 
 extern userProperty* userProperties;
 
-struct _methods;
+struct _jmethodID;
 struct _errorInfo;
 
 /* 64 should put us on the safe side */
@@ -59,7 +66,7 @@ struct _errorInfo;
  * that describes where the parameters are, where the return value should
  * go and what the signature of the method is.
  *
- * `jvalue' is a union defined in include/kaffejtypes.h.  It corresponds
+ * `jvalue' is a union defined in include/jni.h.  It corresponds
  * to an entry on the Java stack. 
  * The suffixes i,j,b,c,s,l,f,d access the corresponding element of
  * (Java) type int, long, boolean, char, short, ref, float, and double.
@@ -96,10 +103,10 @@ struct _errorInfo;
  * sysdepCallMethod must store the return value in the proper type at *ret.
  */
 typedef struct {
-	void*			function;
-	jvalue*			args;
+	void*			function;  /* method information */
+	jvalue*			args; /* treated as an array of method arguments */
 	jvalue*			ret;
-	int			nrargs;
+	int			nrargs; /* number of arguments in args */
 	int			argsize;
 	char			retsize;
 	char			rettype;
@@ -115,8 +122,8 @@ struct Hjava_lang_ClassLoader;
 
 extern const char*		getEngine(void);
 extern void initEngine(void);
-extern void		do_execute_java_method(jvalue *retval, void*, const char*, const char*, struct _methods*, int, ...);
-extern void		do_execute_java_method_v(jvalue *retval, void*, const char*, const char*, struct _methods*, int, va_list);
+extern void		do_execute_java_method(jvalue *retval, void*, const char*, const char*, struct _jmethodID*, int, ...);
+extern void		do_execute_java_method_v(jvalue *retval, void*, const char*, const char*, struct _jmethodID*, int, va_list);
 extern void		do_execute_java_class_method(jvalue *retval, const char*, struct Hjava_lang_ClassLoader*, const char*, const char*, ...);
 extern void		do_execute_java_class_method_v(jvalue *retval, const char*, struct Hjava_lang_ClassLoader*, const char*, const char*, va_list);
 
@@ -124,12 +131,13 @@ extern struct Hjava_lang_Object* execute_java_constructor(const char*, struct Hj
 extern struct Hjava_lang_Object* execute_java_constructor_v(const char*, struct Hjava_lang_ClassLoader*, struct Hjava_lang_Class*, const char*, va_list);
 extern jlong		currentTime(void);
 
-extern void	KaffeVM_callMethodA(struct _methods*, void*, void*, jvalue*, jvalue*, int);
-extern void	KaffeVM_callMethodV(struct _methods*, void*, void*, va_list, jvalue*);
-extern void     KaffeVM_safeCallMethodA(struct _methods*, void*, void*, jvalue*, jvalue*, int);
-extern void     KaffeVM_safeCallMethodV(struct _methods*, void*, void*, va_list, jvalue*);
-extern struct _methods*	lookupClassMethod(struct Hjava_lang_Class*, const char*, const char*, struct _errorInfo*);
-extern struct _methods*	lookupObjectMethod(struct Hjava_lang_Object*, const char*, const char*, struct _errorInfo*);
+extern void	KaffeVM_callMethodA(struct _jmethodID*, void*, void*, jvalue*, jvalue*, int);
+extern void	KaffeVM_callMethodV(struct _jmethodID*, void*, void*, va_list, jvalue*);
+extern void     KaffeVM_safeCallMethodA(struct _jmethodID*, void*, void*, jvalue*, jvalue*, int);
+extern void     KaffeVM_safeCallMethodV(struct _jmethodID*, void*, void*, va_list, jvalue*);
+extern Method*	lookupClassMethod(struct Hjava_lang_Class*, const char*, const char*, struct _errorInfo*);
+extern Method*	lookupObjectMethod(struct Hjava_lang_Object*, const char*, const char*, struct _errorInfo*);
+extern Field*   KNI_lookupFieldC(struct Hjava_lang_Class*, const char*, bool, struct _errorInfo*);
 
 struct _strconst;
 extern void SignalError(const char *, const char *);
