@@ -319,14 +319,14 @@ threadData* jthread_get_data(jthread_t tid);
 int jthreadedOpen(const char* path, int flags, int mode, int*);
 int jthreadedSocket(int af, int type, int proto, int*);
 int jthreadedConnect(int fd, struct sockaddr* addr, int len, int timeout);
-int jthreadedAccept(int fd, struct sockaddr* addr, int* len, 
+int jthreadedAccept(int fd, struct sockaddr* addr, socklen_t* len, 
 	int timeout, int *);
 int jthreadedRead(int fd, void* buf, size_t len, ssize_t *);
 int jthreadedTimedRead(int fd, void* buf, size_t len, int timeout, ssize_t *);
 int jthreadedTimedWrite(int fd, const void* buf, size_t len, int timeout, ssize_t *);
 int jthreadedWrite(int fd, const void* buf, size_t len, ssize_t *);
 int jthreadedRecvfrom(int fd, void* buf, size_t len, int flags,
-        struct sockaddr* from, int* fromlen, int timeout, ssize_t *);
+        struct sockaddr* from, socklen_t* fromlen, int timeout, ssize_t *);
 int jthreadedWaitpid(int wpid, int* status, int options, int *);
 int jthreadedForkExec(char **argv, char **arge,
 		      int ioes[/* 4 */], int *, const char *);
@@ -370,7 +370,7 @@ void jthread_clear_run(jthread_t jt);
 int jthread_has_run(jthread_t jt);
 
 /* let main thread loop until all threads finish, for tests */
-void 	jthread_exit_when_done(void) NONRETURNING;
+void 	jthread_exit_when_done(void);
 
 static inline
 bool jthread_attach_current_thread(UNUSED bool isDaemon)
