@@ -13,12 +13,25 @@
  */
 
 #include "config.h"
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
+#if defined(HAVE_SYS_RESOURCE_H)
+#include <sys/resource.h>
+#endif
+
 #include "debug.h"
 #include "config-std.h"
 #include "config-mem.h"
 #include "config-io.h"
 #include "config-signal.h"
-#include "kaffe/jni_md.h"
+#include "jni_md.h"
 #include "gtypes.h"
 #include "access.h"
 #include "object.h"
@@ -50,9 +63,9 @@ extern struct JNINativeInterface Kaffe_JNINativeInterface;
 static size_t threadStackSize;	/* native stack size */
 
 /* referenced by native/Runtime.c */
-jbool runFinalizerOnExit;	/* should we run finalizers? */
+jboolean runFinalizerOnExit;	/* should we run finalizers? */
 
-static jbool deadlockDetection = 1;	/* abort if we detect deadlock */
+static jboolean deadlockDetection = 1;	/* abort if we detect deadlock */
 
 /**
  * Turn deadlock detection on or off.
@@ -60,7 +73,7 @@ static jbool deadlockDetection = 1;	/* abort if we detect deadlock */
  * @param detect 0 to turn detection off, !0 to turn it on.
  */
 void 
-KaffeVM_setDeadlockDetection(jbool detect) 
+KaffeVM_setDeadlockDetection(jboolean detect) 
 {
   deadlockDetection = detect;
 }
@@ -78,7 +91,7 @@ linkNativeAndJavaThread(jthread_t thread, Hjava_lang_VMThread *jlThread)
 	threadData *thread_data = KTHREAD(get_data)(thread);
 
 	thread_data->jlThread = jlThread;
-	unhand (jlThread)->jthreadID = (struct Hkaffe_util_Ptr *)thread;
+	unhand (jlThread)->vmdata = (struct Hkaffe_util_Ptr *)thread;
 
 	thread_data->jnireferences = NULL;
 	thread_data->jniEnv = &Kaffe_JNINativeInterface;
@@ -120,7 +133,7 @@ initThreads(void)
 	assert(ThreadGroupClass != 0);
 
 	/* Allocate a thread to be the main thread */
-	attachFakedThreadInstance("main", false);
+	KaffeVM_attachFakedThreadInstance("main", false);
 
 	DBG(INIT, dprintf("initThreads() done\n"); );
 }
@@ -185,10 +198,10 @@ void
 interruptThread(Hjava_lang_VMThread* tid)
 {
 DBG(VMTHREAD, dprintf ("%p (%p) interrupting %p (%p)\n", KTHREAD(current)(),
-                       THREAD_DATA()->jlThread, unhand(tid)->jthreadID, tid); );
-	assert(unhand(tid)->jthreadID != NULL);
+                       THREAD_DATA()->jlThread, unhand(tid)->vmdata, tid); );
+	assert(unhand(tid)->vmdata != NULL);
 
-	KTHREAD(interrupt)((jthread_t)unhand(tid)->jthreadID);
+	KTHREAD(interrupt)((jthread_t)unhand(tid)->vmdata);
 }
 
 /*
@@ -206,17 +219,33 @@ stopThread(Hjava_lang_VMThread* tid, Hjava_lang_Object* obj)
 		 * thread won't throw the exception `obj', but it will 
 		 * construct a new ThreadDeath exception when it dies.
 		 */
-		if ((jthread_t)unhand(tid)->jthreadID)
-			KTHREAD(stop)((jthread_t)unhand(tid)->jthreadID);
+		if ((jthread_t)unhand(tid)->vmdata)
+			KTHREAD(stop)((jthread_t)unhand(tid)->vmdata);
 	}
 }
 
-/*
- * Create the initial thread with a given name.
- *  We should only ever call this once.
+/**
+ * Detach this thread from the VM.
+ * This must not be called only when the thread has left the VM
+ * completely (e.g. the native code must not return in the interpreter/JIT
+ * after that).
+ */
+void KaffeVM_detachCurrentThread()
+{
+  Hjava_lang_Thread* tid = getCurrentThread();
+
+  DBG(VMTHREAD, dprintf("detachThreadInstance(%p, %s)\n", tid, nameThread(unhand(tid)->vmThread)); );
+
+  KaffeVM_unlinkNativeAndJavaThread();
+}
+
+/**
+ * Attach an external thread to the VM.
+ * We should only ever call this once for each external thread.
+ * (Note: All assertions on that must have been checked before)
  */
 void
-attachFakedThreadInstance(const char* nm, int isDaemon)
+KaffeVM_attachFakedThreadInstance(const char* nm, int isDaemon)
 {
 	Hjava_lang_Thread* tid;
 	jvalue retval;
@@ -261,7 +290,7 @@ attachFakedThreadInstance(const char* nm, int isDaemon)
         /*
 	 * set context class loader of primordial thread to app classloader
 	 * must not be done earlier, since getCurrentThread() won't work
-         * before the KTHREAD(createfirst) and the jthreadID assignment.
+         * before the KTHREAD(createfirst) and the vmdata assignment.
 	 *
 	 * If we're creating the Thread instance of the main thread, this
 	 * will trigger the initialization process of the java part of the
@@ -422,9 +451,7 @@ firstStartThread(void* arg)
 
 		jvmpiFillThreadStart(&ev, tid);
 		jvmpiPostEvent(&ev);
-		KFREE(ev.u.thread_start.parent_name);
-		KFREE(ev.u.thread_start.group_name);
-		KFREE(ev.u.thread_start.thread_name);
+		jvmpiCleanupThreadStart(&ev);
 	}
 #endif
 
@@ -467,13 +494,13 @@ yieldThread(void)
  * already been updated. In the other case, there will be a small inconsistency.
  */
 void
-setPriorityThread(Hjava_lang_VMThread* tid, int prio)
+setPriorityThread(Hjava_lang_VMThread* tid, jint prio)
 {
 	/* no native thread yet */
-	if (unhand(tid)->jthreadID == 0)
+	if (unhand(tid)->vmdata == 0)
 	        return;
 
-	KTHREAD(setpriority)((jthread_t)unhand(tid)->jthreadID, prio);
+	KTHREAD(setpriority)((jthread_t)unhand(tid)->vmdata, prio);
 }
 
 /*
@@ -537,7 +564,7 @@ getCurrentThread(void)
 void
 finalizeThread(Hjava_lang_VMThread* tid)
 {
-	jthread_t jtid = (jthread_t)unhand(tid)->jthreadID;
+	jthread_t jtid = (jthread_t)unhand(tid)->vmdata;
 
 	if (jtid != NULL) {
 		KTHREAD(destroy)(jtid);
@@ -590,12 +617,9 @@ runfinalizer(void)
 		 * java.lang.Thread instance or it was gc'ed in the mean time,
 		 * create a new one.
 		 */
-		attachFakedThreadInstance("main", false);
+		KaffeVM_attachFakedThreadInstance("main", false);
 	}
 
-	/* Do java-land cleanup */
-	do_execute_java_method(NULL, SystemClass, "exitJavaCleanup",
-			       "()V", NULL, true);
 	if (runFinalizerOnExit) {
 		invokeFinalizer();
 	}
@@ -663,7 +687,7 @@ DBG(VMTHREAD,
 	dprintf("Deadlock: all threads blocked on internal events\n");
 );
 	fflush(stderr);
-	ABORT();
+	KAFFEVM_ABORT();
 }
 
 void
@@ -712,7 +736,9 @@ initNativeThreads(int nativestacksize)
 	  }
 	else if (stackSize == KAFFEMD_STACK_INFINITE || stackSize >= threadStackSize)
 	  {
+#ifndef KAFFEMD_DISABLE_SETSTACKSIZE
 	    mdSetStackSize(threadStackSize);
+#endif
 	    /* Last chance. We check whether the size has really been updated. */
 	    stackSize = mdGetStackSize();
 	  }
