@@ -8,6 +8,7 @@
  * of this file.
  */
 
+#include <limits.h>
 #include <stdio.h>
 
 #include "config.h"
@@ -47,7 +48,7 @@ bi_free(void *ptr, size_t UNUSED size)
 void
 Java_java_math_BigInteger_initialize0(JNIEnv* env, jclass cls)
 {
-	number = (*env)->GetFieldID(env, cls, "number", "kaffe.util.Ptr");
+	number = (*env)->GetFieldID(env, cls, "number", "org.kaffe.util.Ptr");
 	mp_set_memory_functions (bi_alloc, bi_realloc, bi_free);
 }
 
@@ -77,24 +78,42 @@ void
 Java_java_math_BigInteger_assignLong0(JNIEnv* env, jobject r, jlong v)
 {
 	mpz_ptr res;
-	int negative = v < 0 ? -1 : 0;
 
 	res = (*env)->GetObjectField(env, r, number);
 
-	if (negative)
-		v = -v;
-	/* Note that v will remain negative if it's LONG_LONG_MIN.
-	   This is not a problem because any sign copying in the right
-	   shift will be stripped with the cast to jint, and the
-	   number will be considered positive.  Furthermore, in this
-	   case, (jint)v will be zero, so the addition will be a
-	   do-nothing operation.  At last, the number will be made
-	   negative, as appropriate.  */
-	mpz_set_ui(res, (unsigned long)(jint)(v >> 32));
-	mpz_mul_2exp(res, res, 32);
-	mpz_add_ui(res, res, (unsigned long)(jint)v);
-	if (negative)
-		mpz_neg(res, res);
+	/* If a jlong is of the size of a long, we can sinply use the 
+	 * function provided in gmp.
+         */
+	if (sizeof(v) == sizeof(long)) {
+		mpz_set_si(res, v);
+	}
+	else {
+                /* We need to break down v into long sized pieces. Typically
+                 * the case is that a long is 32 bits on a platform, while a jlong
+                 * has 64 bits, so we need to set the upper and lower 32 bits 
+                 * using separate calls to gmp routines.
+                 */
+		const int negative = v < 0 ? -1 : 0;
+		const unsigned int shift_distance = sizeof(long) * CHAR_BIT;
+
+		if (negative)
+			v = -v;
+
+		/* Note that v will remain negative if it's LONG_LONG_MIN.
+		   This is not a problem because any sign copying in the right
+		   shift will be stripped with the cast to jint, and the
+		   number will be considered positive.  Furthermore, in this
+		   case, (jint)v will be zero, so the addition will be a
+		   do-nothing operation.  At last, the number will be made
+		   negative, as appropriate.  */
+
+		mpz_set_ui(res, (unsigned long)((v & 0x7FFFFFFF00000000L) >> shift_distance));
+		mpz_mul_2exp(res, res, shift_distance);
+		mpz_add_ui(res, res, (unsigned long)(v & 0x00000000FFFFFFFFL));
+
+		if (negative)
+			mpz_neg(res, res);
+	}
 }
 
 void
@@ -224,7 +243,8 @@ Java_java_math_BigInteger_rem0(JNIEnv* env, jobject r, jobject s1, jobject s2)
 }
 
 void
-Java_java_math_BigInteger_divrem0(JNIEnv* env, jclass UNUSED cls, jobject r1, jobject r2, jobject s1, jobject s2)
+Java_java_math_BigInteger_divrem0(JNIEnv* env, jclass cls UNUSED, jobject r1,
+				  jobject r2, jobject s1, jobject s2)
 {
 	mpz_ptr res1;
 	mpz_ptr res2;
@@ -474,7 +494,8 @@ Java_java_math_BigInteger_scansetbit0(JNIEnv* env, jobject s)
 
 
 jint
-Java_java_math_BigInteger_cmp0(JNIEnv* env, jclass UNUSED cls, jobject s1, jobject s2)
+Java_java_math_BigInteger_cmp0(JNIEnv* env, jclass cls UNUSED, jobject s1,
+			       jobject s2)
 {
 	mpz_srcptr src1;
 	mpz_srcptr src2;
@@ -559,163 +580,188 @@ Java_java_math_BigInteger_hamDist0(JNIEnv* env, jobject s1, jobject s2)
 static void
 gmp_not_found(JNIEnv* env) 
 {
-	jclass sd = (*env)->FindClass(env, "kaffe.util.SupportDisabled");
+	jclass sd = (*env)->FindClass(env, "org.kaffe.util.SupportDisabled");
 	(*env)->ThrowNew(env, sd, "GNU gmp was not found by Kaffe configure script");
 }
 
 /* We should put some dummies in here */
 void
-Java_java_math_BigInteger_initialize0(JNIEnv* env, jclass cls)
+Java_java_math_BigInteger_initialize0(JNIEnv* env, jclass cls UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_init0(JNIEnv* env, jobject r)
+Java_java_math_BigInteger_init0(JNIEnv* env, jobject r UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_finalize0(JNIEnv* env, jobject r)
+Java_java_math_BigInteger_finalize0(JNIEnv* env, jobject r UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_assignLong0(JNIEnv* env, jobject r, jlong v)
+Java_java_math_BigInteger_assignLong0(JNIEnv* env, jobject r UNUSED,
+				      jlong v UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_assignBytes0(JNIEnv* env, jobject r, jint sign, jbyteArray magnitude)
+Java_java_math_BigInteger_assignBytes0(JNIEnv* env, jobject r UNUSED,
+				       jint sign UNUSED,
+				       jbyteArray magnitude UNUSED)
 {
 	gmp_not_found(env);
 }
 
 jint
-Java_java_math_BigInteger_assignString0(JNIEnv* env, jobject r, jstring val, jint radix)
+Java_java_math_BigInteger_assignString0(JNIEnv* env, jobject r UNUSED,
+					jstring val UNUSED, jint radix UNUSED)
+{
+	gmp_not_found(env);
+
+	return 0;
+}
+
+void
+Java_java_math_BigInteger_add0(JNIEnv* env, jobject r UNUSED, jobject s1 UNUSED,
+			       jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_add0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_sub0(JNIEnv* env, jobject r UNUSED, jobject s1 UNUSED,
+			       jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_sub0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_mul0(JNIEnv* env, jobject r UNUSED, jobject s1 UNUSED,
+			       jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_mul0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_div0(JNIEnv* env, jobject r UNUSED, jobject s1 UNUSED,
+			       jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_div0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_rem0(JNIEnv* env, jobject r UNUSED, jobject s1 UNUSED,
+			       jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_rem0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_divrem0(JNIEnv* env, jclass cls UNUSED,
+				  jobject r1 UNUSED, jobject r2 UNUSED,
+				  jobject s1 UNUSED, jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_divrem0(JNIEnv* env, jclass cls, jobject r1, jobject r2, jobject s1, jobject s2)
+Java_java_math_BigInteger_abs0(JNIEnv* env, jobject r UNUSED, jobject s UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_abs0(JNIEnv* env, jobject r, jobject s)
+Java_java_math_BigInteger_neg0(JNIEnv* env, jobject r UNUSED, jobject s UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_neg0(JNIEnv* env, jobject r, jobject s)
+Java_java_math_BigInteger_pow0(JNIEnv* env, jobject r UNUSED, jobject s UNUSED,
+			       jint power UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_pow0(JNIEnv* env, jobject r, jobject s, jint power)
+Java_java_math_BigInteger_gcd0(JNIEnv* env, jobject r UNUSED, jobject s1 UNUSED,
+			       jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_gcd0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_mod0(JNIEnv* env, jobject r UNUSED, jobject s1 UNUSED,
+			       jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_mod0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_modpow0(JNIEnv* env, jobject r UNUSED,
+				  jobject s1 UNUSED, jobject s2 UNUSED,
+				  jobject s3 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_modpow0(JNIEnv* env, jobject r, jobject s1, jobject s2, jobject s3)
+Java_java_math_BigInteger_modinv0(JNIEnv* env, jobject r UNUSED,
+				  jobject s1 UNUSED, jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_modinv0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_and0(JNIEnv* env, jobject r UNUSED,
+			       jobject s1 UNUSED, jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_and0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_or0(JNIEnv* env, jobject r UNUSED, jobject s1 UNUSED,
+			      jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_or0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_xor0(JNIEnv* env, jobject r UNUSED, jobject s1 UNUSED,
+			       jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_xor0(JNIEnv* env, jobject r, jobject s1, jobject s2)
+Java_java_math_BigInteger_not0(JNIEnv* env, jobject r UNUSED,  jobject s UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_not0(JNIEnv* env, jobject r, jobject s)
+Java_java_math_BigInteger_setbit0(JNIEnv* env, jobject r UNUSED,
+				  jobject s UNUSED,
+				  jint n UNUSED)
 {
 	gmp_not_found(env);
 }
 
 void
-Java_java_math_BigInteger_setbit0(JNIEnv* env, jobject r, jobject s, jint n)
-{
-	gmp_not_found(env);
-}
-
-void
-Java_java_math_BigInteger_clrbit0(JNIEnv* env, jobject r, jobject s, jint n)
+Java_java_math_BigInteger_clrbit0(JNIEnv* env, jobject r UNUSED,
+				  jobject s UNUSED, jint n UNUSED)
 {
 	gmp_not_found(env);
 }
 
 jint
-Java_java_math_BigInteger_scansetbit0(JNIEnv* env, jobject s)
+Java_java_math_BigInteger_scansetbit0(JNIEnv* env, jobject s UNUSED)
 {
 	gmp_not_found(env);
 
@@ -723,7 +769,8 @@ Java_java_math_BigInteger_scansetbit0(JNIEnv* env, jobject s)
 }
 
 jint
-Java_java_math_BigInteger_cmp0(JNIEnv* env, jclass cls, jobject s1, jobject s2)
+Java_java_math_BigInteger_cmp0(JNIEnv* env, jclass cls UNUSED,
+			       jobject s1 UNUSED, jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 
@@ -731,7 +778,8 @@ Java_java_math_BigInteger_cmp0(JNIEnv* env, jclass cls, jobject s1, jobject s2)
 }
 
 jstring
-Java_java_math_BigInteger_toString0(JNIEnv* env, jobject s, jint base)
+Java_java_math_BigInteger_toString0(JNIEnv* env, jobject s UNUSED,
+				    jint base UNUSED)
 {
 	gmp_not_found(env);
 
@@ -739,7 +787,7 @@ Java_java_math_BigInteger_toString0(JNIEnv* env, jobject s, jint base)
 }
 
 jdouble
-Java_java_math_BigInteger_toDouble0(JNIEnv* env, jobject s)
+Java_java_math_BigInteger_toDouble0(JNIEnv* env, jobject s UNUSED)
 {
 	gmp_not_found(env);
 
@@ -747,7 +795,7 @@ Java_java_math_BigInteger_toDouble0(JNIEnv* env, jobject s)
 }
 
 jint
-Java_java_math_BigInteger_toInt0(JNIEnv* env, jobject s)
+Java_java_math_BigInteger_toInt0(JNIEnv* env, jobject s UNUSED)
 {
 	gmp_not_found(env);
 
@@ -755,7 +803,8 @@ Java_java_math_BigInteger_toInt0(JNIEnv* env, jobject s)
 }
 
 jint
-Java_java_math_BigInteger_probablyPrime0(JNIEnv* env, jobject s, jint prop)
+Java_java_math_BigInteger_probablyPrime0(JNIEnv* env, jobject s UNUSED,
+					 jint prop UNUSED)
 {
 	gmp_not_found(env);
 
@@ -763,7 +812,7 @@ Java_java_math_BigInteger_probablyPrime0(JNIEnv* env, jobject s, jint prop)
 }
 
 jint
-Java_java_math_BigInteger_bitLength0(JNIEnv* env, jobject s)
+Java_java_math_BigInteger_bitLength0(JNIEnv* env, jobject s UNUSED)
 {
 	gmp_not_found(env);
 
@@ -771,7 +820,8 @@ Java_java_math_BigInteger_bitLength0(JNIEnv* env, jobject s)
 }
 
 jint
-Java_java_math_BigInteger_hamDist0(JNIEnv* env, jobject s1, jobject s2)
+Java_java_math_BigInteger_hamDist0(JNIEnv* env, jobject s1 UNUSED,
+				   jobject s2 UNUSED)
 {
 	gmp_not_found(env);
 
