@@ -4,13 +4,21 @@
  * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
+ * Copyright (c) 2005
+ *      Kaffe.org contributors.  See ChangeLog for details. 
+ *      All rights reserved.
+ 8
  * See the file "license.terms" for information on usage and redistribution 
  * of this file. 
  */
 
 #include "config.h"
 #include "config-std.h"
-#include "config-std.h"
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
 #include "stringSupport.h"
 #include "java_lang_String.h"
 
@@ -84,7 +92,7 @@ java_lang_String_indexOf(Hjava_lang_String* str, Hjava_lang_String* pat, jint of
 	for ( i=0; i<m; i++ )
 	  bs[ (unsigned char)p[i] ] = m-i;
 
-	k= n - m;
+	k= n - m + 1;
 	for ( i=offset; i < k; ) {
 	  if ( memcmp( &a[i], p, (size_t)m2) == 0 )
 		return i;
